SystemOrganization addCategory: #ExtentionsForKendrick!

TestCase subclass: #TestDependantConcern
	instanceVariableNames: 'beforeSimulator afterSimulator beforeModel afterModel beforeDiagram afterDiagram'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestDependantConcern methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/17/2016 18:50'!
after

KendrickModel OneSpeciesSpatial	
	population: 5000.

Concern Spatial
	attribute: #(patch -> 1  2  3  4  5);
	parameters: #(rho);
	rho: 0.03;
	transitions: #(
		1 -- rho --> 2.
		2 -- rho --> 3.
		3 -- rho --> 4.
		4 -- rho --> 5.
		5 -- rho --> 1.
	).
		
Concern SIR
	attribute: #(status -> S I R);
	parameters: #(beta lambda sigma gamma mu);
	mu: 0.0000365;
	gamma: 0.25;
	lambda: #(beta*I/N);
	transitions: #(
		S -- lambda --> I.
		I -- gamma --> R.
	).

Concern SEIR
	extends: 'SIR';
	parameters: #(sigma);
	sigma: 0.5;
	delay: #(sigma , S -- lambda --> I  , E).
	
Composition OneSpatialCombo
	model: 'OneSpeciesSpatial';
	concern: 'Spatial';
	concern: 'SEIR';
	beta_patch: #(0.75 0.5 0.5 0.5 0.5);
	N: #(patch);
	lambda: #(beta*I_patch/N); "check how this should work"
	S_patch: #(900 1000 1000 1000 1000);
	I_patch: #(100 0 0 0 0).
		
Simulation SpatialRungeKutta rungeKutta
	for: 'OneSpatialCombo';
	from: 0; 
	to: 500; 
	step: 1.
	
Visualization SpatialDiagramViz diagram 
	for: 'SpatialRungeKutta';
	data: #(I_patch);
	legends: #('patch 1' 'patch 2' 'patch 3' 'patch 4' 'patch 5');
	legendTitle: 'Infectious';
	xLabel: 'Time (days)'.! !

!TestDependantConcern methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 21:33'!
afterBench

KendrickModel OneSpeciesSpatial	
	population: 5000.

Concern Spatial
	attribute: #(patch -> 1  2  3  4  5);
	parameters: #(rho);
	rho: 0.03;
	transitions: #(
		1 -- rho --> 2.
		2 -- rho --> 3.
		3 -- rho --> 4.
		4 -- rho --> 5.
		5 -- rho --> 1.
	).
		
Concern SIR
	attribute: #(status -> S I R);
	parameters: #(beta lambda sigma gamma mu);
	mu: 0.0000365;
	gamma: 0.25;
	lambda: #(beta*I/N);
	transitions: #(
		S -- lambda --> I.
		I -- gamma --> R.
	).

Concern SEIR
	extends: 'SIR';
	parameters: #(sigma);
	sigma: 0.5;
	delay: #(sigma , S -- lambda --> I  , E).
	
Composition OneSpatialCombo
	model: 'OneSpeciesSpatial';
	concern: 'Spatial';
	concern: 'SEIR';
	beta_patch: #(0.75 0.5 0.5 0.5 0.5);
	N: #(patch);
	lambda: #(beta*I_patch/N); "check how this should work"
	S_patch: #(900 1000 1000 1000 1000);
	I_patch: #(100 0 0 0 0).
		
Simulation SpatialRungeKutta rungeKutta
	for: 'OneSpatialCombo';
	from: 0; 
	to: 500; 
	step: 1;
	init;
	execute.! !

!TestDependantConcern methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 19:34'!
before

|model sirConcern eConcern spatialConcern simulator db|
sirConcern := KEConcern new.
sirConcern attributes: {#status->#(#S #I #R)}.
sirConcern addParameters: {#lambda. #gamma}.
sirConcern
    addTransitionFrom: '{#status: #S}'
    to: '{#status: #I}'
    probability: [ :aModel | aModel atParameter: #lambda ].
sirConcern 
    addTransitionFrom: '{#status: #I}' 
    to: '{#status: #R}' 
    probability: [ :aModel | aModel atParameter: #gamma ].

spatialConcern := KEConcern new.
spatialConcern addAttribute: #patch value: (1 to: 5) asArray.
spatialConcern addParameter: #rho.
(1 to: 5) do: [ :i| 
    (i < 5) 
    ifTrue: [  
        spatialConcern 
            addTransitionFrom: { #patch->i } 
            to: { #patch->(i+1) } 
            probability: [ :aModel| aModel atParameter: #rho ].
    ]
    ifFalse: [  
        spatialConcern 
            addTransitionFrom: { #patch->i } 
            to: { #patch->1 } 
            probability: [ :aModel| aModel atParameter: #rho ].
    ]
]. 

model := KEModel new.
model population: (KEPopulation size: 5000).
model integrate: spatialConcern.
model integrate: sirConcern.

eConcern := KEConcern dependOn: sirConcern.
eConcern atAttribute: #status add: #E.
eConcern 
    removeTransitionFrom: { #status->#S } 
    to: { #status->#I }.
eConcern 
    addTransitionFrom: { #status->#S } 
    to: { #status->#E } 
    probability: [ :aModel| aModel atParameter: #lambda ].
eConcern 
    addTransitionFrom: { #status->#E } 
    to: { #status->#I } 
    probability: [ :aModel| aModel atParameter: #sigma ].
eConcern 
    addTransitionFrom: { #status->#E } 
    to: #empty 
    probability: [ :aModel| aModel atParameter: #mu ].
eConcern addParameter: #sigma.
model integrate: eConcern.

model atParameter: #beta assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #patch.
        c = 1 ifTrue: [ val := 0.75 ] ifFalse: [ val := 0.5 ].
        val 
    ].
model atParameter: #mu assignValue: 0.0000365.
model atParameter: #sigma assignValue: 0.5.
model atParameter: #gamma assignValue: 0.25.
model atParameter: #N assignValue: [ :aModel| |c| 
    c := aModel currentCompartment at: #patch.
    aModel sizeOfPopulation: c
    ].
model atParameter: #rho assignValue: 0.03. 
model atParameter: #lambda assignValue: [ :aModel| |c|
    c := aModel currentCompartment at: #patch.
    (aModel atParameter: #beta) * (aModel atCompartment: {#status->#I. #patch->c}) / (aModel atParameter: #N)
    ].
    model atCompartment: { #status->#S. #patch->1 } put: 900 atOthersPut: 0.
    model atCompartment: { #status->#I. #patch->1 } put: 100.
    model atCompartment: { #status->#S. #patch->2 } put: 1000.
    model atCompartment: { #status->#S. #patch->3 } put: 1000.
    model atCompartment: { #status->#S. #patch->4 } put: 1000.
    model atCompartment: { #status->#S. #patch->5 } put: 1000.

    simulator := KESimulator new: #RungeKutta from: 0 to: 500 step: 1.
    simulator executeOn: model.

    db := KEDiagramBuilder new.
    db data: [|tmp| tmp := OrderedCollection new.
    tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:1}').
    tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:2}').
    tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:3}').
    tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:4}').
    tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:5}').
    tmp] value.
    db legends: { 'patch 1'. 'patch 2'. 'patch 3'. 'patch 4'. 'patch 5'}.
    db legendTitle: 'Infectious'.
    db xLabel: 'Time (days)'.
    db open.

	^ {simulator . model . db}! !

!TestDependantConcern methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 21:33'!
beforeBench

|model sirConcern eConcern spatialConcern simulator db|
sirConcern := KEConcern new.
sirConcern attributes: {#status->#(#S #I #R)}.
sirConcern addParameters: {#lambda. #gamma}.
sirConcern
    addTransitionFrom: '{#status: #S}'
    to: '{#status: #I}'
    probability: [ :aModel | aModel atParameter: #lambda ].
sirConcern 
    addTransitionFrom: '{#status: #I}' 
    to: '{#status: #R}' 
    probability: [ :aModel | aModel atParameter: #gamma ].

spatialConcern := KEConcern new.
spatialConcern addAttribute: #patch value: (1 to: 5) asArray.
spatialConcern addParameter: #rho.
(1 to: 5) do: [ :i| 
    (i < 5) 
    ifTrue: [  
        spatialConcern 
            addTransitionFrom: { #patch->i } 
            to: { #patch->(i+1) } 
            probability: [ :aModel| aModel atParameter: #rho ].
    ]
    ifFalse: [  
        spatialConcern 
            addTransitionFrom: { #patch->i } 
            to: { #patch->1 } 
            probability: [ :aModel| aModel atParameter: #rho ].
    ]
]. 

model := KEModel new.
model population: (KEPopulation size: 5000).
model integrate: spatialConcern.
model integrate: sirConcern.

eConcern := KEConcern dependOn: sirConcern.
eConcern atAttribute: #status add: #E.
eConcern 
    removeTransitionFrom: { #status->#S } 
    to: { #status->#I }.
eConcern 
    addTransitionFrom: { #status->#S } 
    to: { #status->#E } 
    probability: [ :aModel| aModel atParameter: #lambda ].
eConcern 
    addTransitionFrom: { #status->#E } 
    to: { #status->#I } 
    probability: [ :aModel| aModel atParameter: #sigma ].
eConcern 
    addTransitionFrom: { #status->#E } 
    to: #empty 
    probability: [ :aModel| aModel atParameter: #mu ].
eConcern addParameter: #sigma.
model integrate: eConcern.

model atParameter: #beta assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #patch.
        c = 1 ifTrue: [ val := 0.75 ] ifFalse: [ val := 0.5 ].
        val 
    ].
model atParameter: #mu assignValue: 0.0000365.
model atParameter: #sigma assignValue: 0.5.
model atParameter: #gamma assignValue: 0.25.
model atParameter: #N assignValue: [ :aModel| |c| 
    c := aModel currentCompartment at: #patch.
    aModel sizeOfPopulation: c
    ].
model atParameter: #rho assignValue: 0.03. 
model atParameter: #lambda assignValue: [ :aModel| |c|
    c := aModel currentCompartment at: #patch.
    (aModel atParameter: #beta) * (aModel atCompartment: {#status->#I. #patch->c}) / (aModel atParameter: #N)
    ].
    model atCompartment: { #status->#S. #patch->1 } put: 900 atOthersPut: 0.
    model atCompartment: { #status->#I. #patch->1 } put: 100.
    model atCompartment: { #status->#S. #patch->2 } put: 1000.
    model atCompartment: { #status->#S. #patch->3 } put: 1000.
    model atCompartment: { #status->#S. #patch->4 } put: 1000.
    model atCompartment: { #status->#S. #patch->5 } put: 1000.

    simulator := KESimulator new: #RungeKutta from: 0 to: 500 step: 1.
    simulator executeOn: model! !

!TestDependantConcern methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/20/2016 13:31'!
bench

	| before after |
	
	before := self benchmark: [self beforeBench] times: 100.
	after := self benchmark: [self afterBench] times: 100.
	
	Transcript 
		show: before; cr; 
		show: after; cr; 
		show: (after at: 1) / (before at: 1); cr; cr.
	! !

!TestDependantConcern methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 21:14'!
benchmark: aBlock times: anInteger

	"µ the average time in ms and τ standard deviation"

	| runs average deviation |

	runs := [ 
		(1 to: anInteger) 
			collect: [ :each | Time millisecondsToRun: aBlock ] ] 
		valueUnpreemptively.
	average := runs average asFloat.
	deviation := ((runs 
		detectSum: [ :each | (each - average) * (each - average) ]) 
			/ runs size) sqrt.
	^ Array with: average with: deviation! !

!TestDependantConcern methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 19:35'!
createModels

	| beforeList |
	beforeList := self before.
	beforeSimulator := beforeList first.
	beforeModel := beforeList second.
	beforeDiagram := beforeList third.
	self after.
	afterSimulator := Simulation withId: 'SpatialRungeKutta'.
	afterModel := KendrickModel withId: 'OneSpeciesSpatial'.
	afterDiagram := (Visualization withId: 'SpatialDiagramViz') open.	! !

!TestDependantConcern methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/23/2016 18:20'!
setUp

	KendrickInternalDSLEntities clearAll.
	"KEGillespieSimulatorForTesting newSeed." "stochastic but reproducible each time"! !

!TestDependantConcern methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/23/2016 18:20'!
tearDown

	KendrickInternalDSLEntities clearAll.! !

!TestDependantConcern methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 20:42'!
testEquivalence


	| resultsBefore resultsAfter |
	self createModels.

	beforeModel inspect.
	afterModel keModel inspect.

	"correct (check inspector) -- but timeseries are presented in different order fix"

	resultsBefore := (beforeDiagram data collect: [ :ts | ts peakOfEpidemic floor ]) sorted asOrderedCollection.
	resultsAfter := (afterDiagram data collect: [ :ts | ts peakOfEpidemic floor ]) sorted asOrderedCollection.
	Transcript show: resultsBefore; cr; show: resultsAfter; cr; cr.
	self assert: resultsBefore = resultsAfter.! !

TestCase subclass: #TestEntitiesOnDisk
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestEntitiesOnDisk methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/31/2016 20:23'!
setUp

	KendrickInternalDSLEntities clearAll.

	
	
! !

!TestEntitiesOnDisk methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/31/2016 20:24'!
tearDown

	KendrickInternalDSLEntities clearAll.

	
	
! !

!TestEntitiesOnDisk methodsFor: 'as yet unclassified' stamp: 'BuiThiMaiAnh 6/1/2016 09:28'!
testInfluenza

	"ensure that you have the folder structure in the same dir as the image and un-comment before running"
	
	(Visualization withId: 'DiagramVizForInfluenza') open.

	
	
! !

TestCase subclass: #TestMeaslesScripts
	instanceVariableNames: 'beforeSimulator afterSimulator beforeModel afterModel beforeDiagram afterDiagram'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/17/2016 18:50'!
after

KendrickModel	Measles
	attribute: #(status -> S E I R);
	parameters: #(beta gamma mu sigma); 
	equations: #(
	   S:t=mu*N - beta*S*I - mu*S.
      E:t=beta*S*I - sigma*E - mu*E.
      I:t=sigma*E - gamma*I - mu*I.
      R:t=gamma*I - mu*R.
	);
	population: 100000;
	S: 99999;
	I: 1;
	others: 0;
	beta: 0.0000214;
	gamma: 0.143;
	mu: 0.0000351;
	sigma: 0.125.	
		
Simulation MeaslesRKSim rungeKutta
	forModel: 'Measles';
	from: 0.0; 
	to: 150; 
	step: 1.
	
Visualization MeaslesDiagramViz diagram 
	for: 'MeaslesRKSim';
	xLabel: 'Time (days)'.! !

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 21:48'!
afterBench

KendrickModel	Measles
	attribute: #(status -> S E I R);
	parameters: #(beta gamma mu sigma); 
	equations: #(
	   S:t=mu*N - beta*S*I - mu*S.
      E:t=beta*S*I - sigma*E - mu*E.
      I:t=sigma*E - gamma*I - mu*I.
      R:t=gamma*I - mu*R.
	);
	population: 100000;
	S: 99999;
	I: 1;
	others: 0;
	beta: 0.0000214;
	gamma: 0.143;
	mu: 0.0000351;
	sigma: 0.125.	
		
Simulation MeaslesRKSim rungeKutta
	forModel: 'Measles';
	from: 0.0; 
	to: 150; 
	step: 1;
	init;
	execute.! !

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 20:27'!
before

| model simulator diag |
model := KEModel new population: (KEPopulation size: 100000).
model addAttribute: #status value: #(S E I R).
model atCompartment: { #status->#S } put: 99999 atOthersPut: 0.
model atCompartment: { #status->#I } put: 1.
model addEquations: { 
            'S:t=mu*N - beta*S*I - mu*S'.
            'E:t=beta*S*I - sigma*E - mu*E'.
            'I:t=sigma*E - gamma*I - mu*I'.
            'R:t=gamma*I - mu*R'
            }.
model addParameters: {
        #beta->0.0000214. 
        #gamma->0.143. 
        #mu->0.0000351. 
        #sigma->0.125 }.
simulator := KESimulator new: #RungeKutta from: 0.0 to: 150 step: 1.
simulator executeOn: model.
diag := (KEDiagramBuilder new) data: simulator allTimeSeries.
diag xLabel: 'Time (days)'.
diag open.

^ {simulator . model . diag}! !

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 21:48'!
beforeBench

| model simulator diag |
model := KEModel new population: (KEPopulation size: 100000).
model addAttribute: #status value: #(S E I R).
model atCompartment: { #status->#S } put: 99999 atOthersPut: 0.
model atCompartment: { #status->#I } put: 1.
model addEquations: { 
            'S:t=mu*N - beta*S*I - mu*S'.
            'E:t=beta*S*I - sigma*E - mu*E'.
            'I:t=sigma*E - gamma*I - mu*I'.
            'R:t=gamma*I - mu*R'
            }.
model addParameters: {
        #beta->0.0000214. 
        #gamma->0.143. 
        #mu->0.0000351. 
        #sigma->0.125 }.
simulator := KESimulator new: #RungeKutta from: 0.0 to: 150 step: 1.
simulator executeOn: model.
! !

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/20/2016 13:31'!
bench

	| before after |
	
	before := self benchmark: [self beforeBench] times: 100.
	after := self benchmark: [self afterBench] times: 100.
	
	Transcript 
		show: before; cr; 
		show: after; cr; 
		show: (after at: 1) / (before at: 1); cr; cr.! !

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 21:13'!
benchmark: aBlock times: anInteger

	"µ the average time in ms and τ standard deviation"

	| runs average deviation |

	runs := [ 
		(1 to: anInteger) 
			collect: [ :each | Time millisecondsToRun: aBlock ] ] 
		valueUnpreemptively.
	average := runs average asFloat.
	deviation := ((runs 
		detectSum: [ :each | (each - average) * (each - average) ]) 
			/ runs size) sqrt.
	^ Array with: average with: deviation! !

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 20:33'!
createModels

	| beforeList |
	beforeList := self before.
	beforeSimulator := beforeList first.
	beforeModel := beforeList second.
	beforeDiagram := beforeList third.
	self after.
	afterSimulator := Simulation withId: 'MeaslesRKSim'.
	afterModel := KendrickModel withId: 'Measles'.
	afterDiagram := (Visualization withId: 'MeaslesDiagramViz') open.! !

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 20:33'!
setUp

	KendrickInternalDSLEntities clearAll.
	"KEGillespieSimulatorForTesting newSeed." "stochastic but reproducible each time"! !

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:34'!
tearDown

	KendrickInternalDSLEntities clearAll.! !

!TestMeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 20:41'!
testEquivalence


	| resultsBefore resultsAfter |
	self createModels.

	beforeModel inspect.
	afterModel keModel inspect.

	"correct (check inspector) -- but timeseries are presented in different order fix"

	resultsBefore := (beforeDiagram data collect: [ :ts | ts peakOfEpidemic floor ]) sorted asOrderedCollection.
	resultsAfter := (afterDiagram data collect: [ :ts | ts peakOfEpidemic floor ]) sorted asOrderedCollection.
	Transcript show: resultsBefore; cr; show: resultsAfter; cr; cr.
	self assert: resultsBefore = resultsAfter.! !

TestCase subclass: #TestMosquitoScripts
	instanceVariableNames: 'beforeSimulator afterSimulator beforePair beforeModel afterModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestMosquitoScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/17/2016 18:46'!
after

KendrickModel Mosquito
	population: 13000;
	attribute: #(status -> S I R); 
	parameters: #(mu gamma beta lambda rho);
	mu: 12.17;
	gamma: 52;
	beta: 1;
	lambda: #(beta*I);
	transitions: #(
		S -- lambda --> I.
		I -- gamma --> R.
		status -- mu --> Empty.
		Empty -- mu --> S.
	).

Concern SpeciesConcern
	attribute: #(species -> mosquito reservoir1 reservoir2).
		
Composition MultiMosquito
	model: 'Mosquito';
	concern: 'SpeciesConcern';	
	S_species: #(9999 1000 2000);
	I_species: #(1 0 0);
	"others: 0;"
	N: #(species);
	mu_species: #(12.17 0.05 0.05);
	rho_species: #(
		#(0 0.02 0.02) 
		#(0.02 0 0) 
		#(0.02 0 0)
	);
	lambda: #(beta*rho*I sum).

Simulation MosquitoGillepsie gillespie
	for: 'MultiMosquito';
	from: 0.0; 
	to: 0.5; 
	step: 0.0027.
	
Visualization MosquitoDiagramViz diagram 
	for: 'MosquitoGillepsie';
	data: #(I sqrt);
	xLabel: 'Time (days)'.! !

!TestMosquitoScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/12/2016 19:09'!
before

| model multiHostConcern simulator db |
model := KEModel new.
model population: (KEPopulation size: 13000).
model attributes: {(#status -> #(#S #I #R))}.
model addParameter: #mu value: 12.17.
model addParameter: #gamma value: 52.
model addParameter: #beta value: 1.
model addParameter: #lambda value: [ :aModel| 
        (aModel atParameter: #beta) * 
        (aModel atCompartment: {#status->#I}) ].
model
   addTransitionFrom: '{#status: #S}'
   to: '{#status: #I}'
   probability: [ :m | m atParameter: #lambda ].
model 
   addTransitionFrom: '{#status: #I}' 
   to: '{#status: #R}' 
        probability: [ :m | m atParameter: #gamma ].
    #(#S #I #R) do: [ :each| 
        model 
            addTransitionFrom: {#status->each} 
            to: #empty 
            probability: [ :m | m atParameter: #mu ].
        ].
model 
        addTransitionFrom: #empty 
        to: '{#status: #S}' 
        probability: [ :m | m atParameter: #mu ].

multiHostConcern := KEConcern new.
multiHostConcern 
   addAttribute: #species 
   value: #(#mosquito #reservoir1 #reservoir2).

model integrate: multiHostConcern.

model 
        atParameter: #mu
        assignValue: 
        [ :aModel| |c val| 
            c := aModel currentCompartment at: #species.
            c = #mosquito ifTrue: [ val := 12.17 ].
            c = #reservoir1 ifTrue: [ val := 0.05 ].
            c = #reservoir2 ifTrue: [ val := 0.05 ].
            val
        ].
model atParameter: #N assignValue: [ :aModel| |c| 
        c := aModel currentCompartment at: #species.
        aModel sizeOfPopulation: c
        ].
model addParameter: #rho value: [ :aModel| |c val|
        c := aModel currentCompartment at: #species.
        c = #mosquito ifTrue: [ val := #(0 0.02 0.02) ].
        c = #reservoir1 ifTrue: [ val := #(0.02 0 0) ].
        c = #reservoir2 ifTrue: [ val := #(0.02 0 0) ].
        val
        ].
model atParameter: #lambda assignValue: [ :aModel|
        ((aModel atParameter: #beta) * 
        (aModel atParameter: #rho) * 
        (aModel atCompartment: {#status->#I})) sum
        ].

    model atCompartment: { #status->#I. #species->#mosquito } put: 1 atOthersPut: 0.
    model atCompartment: { #status->#S. #species->#mosquito } put: 9999.
    model atCompartment: { #status->#S. #species->#reservoir1 } put: 1000.
    model atCompartment: { #status->#S. #species->#reservoir2 } put: 2000.

simulator := KESimulator new: #Gillespie from: 0.0 to: 0.5 step: 0.0027.

simulator executeOn: model.
db := (KEDiagramBuilder new) data: ((simulator timeSeriesAt: '{#status: #I}') sqrt).
db open.

^ simulator -> model! !

!TestMosquitoScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/25/2016 10:44'!
createModels

	beforePair := self before.
	beforeSimulator := beforePair key.
	beforeModel := beforePair value.
	self after.
	afterSimulator := Simulation withId: 'MosquitoGillepsie'.
	afterModel := KendrickModel withId: 'Mosquito'.
	(Visualization withId: 'MosquitoDiagramViz') open.	! !

!TestMosquitoScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/18/2016 18:27'!
setUp

	KendrickInternalDSLEntities clearAll.
	"KEGillespieSimulatorForTesting newSeed." "stochastic but reproducible each time"
	! !

!TestMosquitoScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/6/2016 16:50'!
tearDown

	KendrickInternalDSLEntities clearAll.! !

!TestMosquitoScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 20:36'!
testEquivalence

	self createModels.

	beforeModel inspect.
	afterModel keModel inspect.

	"gillepsie is stochastic -- need reproducible stochastic sim to properly compare"! !

TestCase subclass: #TestMultiSpeciesSpatial
	instanceVariableNames: 'beforeSimulator afterSimulator beforeModel afterModel beforeDiagram afterDiagram'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestMultiSpeciesSpatial methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/17/2016 18:45'!
after

KendrickModel MultiSpeciesSpatial	
	population: 27500.
	
Concern MultiHost
	attribute: #(species -> human bird).
	"parameters: #(delta)."
	
Concern Spatial
	attribute: #(patch -> 1  2  3  4  5);
	parameters: #(rho);
	rho: 0.03;
	transitions: #(
		1 -- rho --> 2.
		2 -- rho --> 3.
		3 -- rho --> 4.
		4 -- rho --> 5.
		5 -- rho --> 1.
	).
		
Concern Epidemiological
	attribute: #(status -> S  E  I  R);
	parameters: #(beta lambda sigma gamma v mu);
	v: 0.00274;
	"mu: 0.0000365;"
	"sigma: 0.5;"
	gamma: 0.25;
	"beta: 1;"
	"lambda: #(beta*I/N);"
	transitions: #(
		S -- lambda --> E.
		E -- sigma --> I.
		I -- gamma --> R.
		R -- v --> S.
		status -- mu --> Empty.
		Empty -- mu --> S.
	).
		
Composition MultiSpatialCombo
	model: 'MultiSpeciesSpatial';
	concern: 'Spatial';
	concern: 'MultiHost';
	concern: 'Epidemiological';
	"beta_patch: #(0.75 0.5 0.5 0.5 0.5);"
	mu_species: #(0.0000365 0.00137);
	sigma_species: #(0.5 0.67);
	gamma_species: #(0.25 0.233);
	N: #(patch_species);
	rho_species: #(0.03 0.1);
	beta_species: #(
		#(0 0.21)
		#(0 0.42)
	);
	"delta_species: #(
		#(0 0.21)
		#(0 0.42)
	);"
	lambda: #(beta*I_patch/N sum); "check how this should work"
	S_species_patch: #(
		#(500 500 500 500 500)
		#(4990 5000 5000 5000 5000)
	);
	I_species_patch: #(
		#(0 0 0 0 0)
		#(10 0 0 0 0)
	).
		
Simulation SpatialRungeKutta rungeKutta
	for: 'MultiSpatialCombo';
	from: 0; 
	to: 500; 
	step: 1.
	
Visualization SpatialDiagramViz diagram 
	for: 'SpatialRungeKutta';
	data: #(I_species);"seems to be need some more handling here"
	legendTitle: 'Total of Infectious';
	legends: #('humans' 'birds');
	xLabel: 'Time (days)'.! !

!TestMultiSpeciesSpatial methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 19:17'!
before

|model sirConcern eConcern spatialConcern simulator db multiHostConcern seirsConcern f |
model := KEModel new population: (KEPopulation size: 27500).

"Multi-host concern"
multiHostConcern := KEConcern new.
multiHostConcern
   addAttribute: #species 
   value: #(#human #bird).

"Spatial concern"
spatialConcern := KEConcern new.
spatialConcern addAttribute: #patch value: (1 to: 5) asArray.
spatialConcern addParameter: #rho.
    (1 to: 5) do: [ :i| 
        (i < 5) 
        ifTrue: [  
            spatialConcern 
                addTransitionFrom: { #patch->i } 
                to: { #patch->(i+1) } 
                probability: [ :aModel| aModel atParameter: #rho ].
        ]
        ifFalse: [  
            spatialConcern 
                addTransitionFrom: { #patch->i } 
                to: { #patch->1 } 
                probability: [ :aModel| aModel atParameter: #rho ].
        ]
    ]. 

"SEIRS concern"
seirsConcern := KEConcern new.
seirsConcern attributes: {#status->#(#S #E #I #R)}.
seirsConcern addParameters: { #beta. #gamma. #mu. #sigma. #nu }.
seirsConcern addParameter: #lambda value: [ :aModel | 
            (aModel atParameter: #beta) * 
            (aModel atCompartment: {#status->#I}) / (aModel atParameter: #N) ].
seirsConcern
        addTransitionFrom: '{#status: #S}'
        to: '{#status: #E}'
        probability: [ :aModel | 
            (aModel atParameter: #lambda) ].
    seirsConcern 
        addTransitionFrom: '{#status: #E}' 
        to: '{#status: #I}' 
        probability: [ :aModel | aModel atParameter: #sigma ].
    seirsConcern 
        addTransitionFrom: '{#status: #I}' 
        to: '{#status: #R}' 
        probability: [ :aModel | aModel atParameter: #gamma ].
    seirsConcern 
        addTransitionFrom: '{#status: #R}' 
        to: '{#status: #S}' 
        probability: [ :aModel | aModel atParameter: #nu ].
    #(S E I R) do: [ :each| 
        seirsConcern 
            addTransitionFrom: {#status->each}
            to: #empty 
            probability: [ :aModel | aModel atParameter: #mu ].
        ].
    seirsConcern 
        addTransitionFrom: #empty 
        to: '{#status: #S}' 
        probability: [ :aModel | aModel atParameter: #mu ].

"Concerns integration"

    model integrate: spatialConcern.
    model integrate: multiHostConcern.
    model integrate: seirsConcern.

    model atParameter: #mu assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #species.
        c = #human ifTrue: [ val := 0.0000365 ].
        c = #bird ifTrue: [ val := 0.00137 ].
        val ].
    model atParameter: #sigma assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #species.
        c = #human ifTrue: [ val := 0.5 ].
        c = #bird ifTrue: [ val := 0.67 ].
        val ].
    model atParameter: #gamma assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #species.
        c = #human ifTrue: [ val := 0.25 ].
        c = #bird ifTrue: [ val := 0.233 ].
        val ].
    model atParameter: #nu assignValue: 0.00274.
    model atParameter: #N assignValue: [ :aModel| |c| 
        c := OrderedCollection new.
        c add: (aModel currentCompartment at: #patch).
        c add: (aModel currentCompartment at: #species).
        aModel sizeOfPopulation: (c asArray)
    ].
    model atParameter: #rho assignValue: [ :aModel| |c val|
        c := aModel currentCompartment at: #species.
        c = #bird ifTrue: [ val := 0.1 ].
        c = #human ifTrue: [ val := 0.03 ].
        val
    ].
    model atParameter: #beta assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #species.
        c = #human ifTrue: [ val := #(0 0.21) ].
        c = #bird ifTrue: [ val := #(0 0.42) ].
        val
    ]. 
    model atParameter: #lambda assignValue: [ :aModel| |c|
        c := aModel currentCompartment at: #patch.
        ((aModel atParameter: #beta) *
        (aModel atCompartment: {#status->#I. #patch->c}) / (aModel atParameter: #N)) sum.
    ].
    model 
        atCompartment: { #status->#S. #species->#bird. #patch->1 } 
        put: 4990
        atOthersPut: 0.
    model 
        atCompartment: { #status->#I. #species->#bird. #patch->1 } 
        put: 10.
    2 to: 5 do: [ :i| 
        model 
            atCompartment: { #status->#S. #species->#bird. #patch->i } 
            put: 5000.
    ].
    1 to: 5 do: [ :i| 
        model 
            atCompartment: { #status->#S. #species->#human. #patch->i } 
            put: 500 
    ].

"Run the model with RK4"
simulator := KESimulator new: #RungeKutta from: 0 to: 500 step: 1.
    simulator executeOn: model.
    db := KEDiagramBuilder new.

    f := [:name| |d tmp|
        tmp := (simulator timeSeriesAt: name) collect: [ :e| e value ].
        d := OrderedCollection new.
        1 to: tmp first data size do: [ :k| d add: (tmp collect: [:e| e data at: k]) sum ].
        (KETimeSeries from: d withIndex: tmp first index) compartment: (STON fromString: name)
        ].
    db data: { 
        (f value: '{#status:#I, #species:#bird}'). 
        (f value: '{#status: #I, #species: #human}') }.
    db xLabel: 'Time (days)'.
    db legendTitle: 'Total of infectious'.
    db legends: { 'birds'. 'humans' }.

    db open.

	^ { simulator . model . db}! !

!TestMultiSpeciesSpatial methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 19:18'!
createModels

	| beforeList |
	beforeList := self before.
	beforeSimulator := beforeList first.
	beforeModel := beforeList second.
	beforeDiagram := beforeList third.
	self after.
	afterSimulator := Simulation withId: 'SpatialRungeKutta'.
	afterModel := KendrickModel withId: 'MultiSpeciesSpatial'.
	afterDiagram := (Visualization withId: 'SpatialDiagramViz') open.	! !

!TestMultiSpeciesSpatial methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/23/2016 18:23'!
setUp

	KendrickInternalDSLEntities clearAll.
	"KEGillespieSimulatorForTesting newSeed." "stochastic but reproducible each time"! !

!TestMultiSpeciesSpatial methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/23/2016 18:23'!
tearDown

	KendrickInternalDSLEntities clearAll.! !

!TestMultiSpeciesSpatial methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 20:40'!
testEquivalence


	| resultsBefore resultsAfter |
	self createModels.

	beforeModel inspect.
	afterModel keModel inspect.

	"correct (check inspector) -- but timeseries are presented in different order fix"

	resultsBefore := (beforeDiagram data collect: [ :ts | ts peakOfEpidemic floor ]) sorted asOrderedCollection.
	resultsAfter := (afterDiagram data collect: [ :ts | ts peakOfEpidemic floor ]) sorted asOrderedCollection.
	Transcript show: resultsBefore; cr; show: resultsAfter; cr; cr.
	self assert: resultsBefore = resultsAfter.! !

TestCase subclass: #TestMultiSpeciesSpatialDependant
	instanceVariableNames: 'beforeSimulator afterSimulator beforeModel afterModel beforeDiagram afterDiagram'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestMultiSpeciesSpatialDependant methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 20:16'!
after

KendrickModel Infuenza.
	
Concern Biological
	attribute: #(species -> human bird).
	
Concern Demographical
	attribute: #(patch -> 1  2  3  4  5);
	parameters: #(rho);
	rho: 0.03;
	transitions: #(
		1 -- rho --> 2.
		2 -- rho --> 3.
		3 -- rho --> 4.
		4 -- rho --> 5.
		5 -- rho --> 1.
	).
		
Concern SIR
	attribute: #(status -> S I R);
	parameters: #(beta lambda gamma mu v);
	gamma: 0.25;
	v: 0.00274;
	transitions: #(
		S -- lambda --> I.
		I -- gamma --> R.
		R -- v --> S.
	).	
		
Concern SEIR
	extends: 'SIR';
	parameters: #(sigma);
	addTransition:	#(Empty -- mu --> S.);
	addTransition: #(status -- mu --> Empty.);
	delay: #(sigma , S -- lambda --> I  , E).
		
Composition MultiSpeciesSpatial
	model: 'Infuenza';
	populationSize: 27500;
	concern: 'Demographical';
	concern: 'Biological';
	concern: 'SEIR';
	mu_species: #(0.0000365 0.00137);
	sigma_species: #(0.5 0.67);
	gamma_species: #(0.25 0.233);
	rho_species: #(0.03 0.1);
	beta_species: #(
		#(0 0.21)
		#(0 0.42)
	);
	lambda: #(beta*I_patch/N sum);
	N: #(patch_species);
	S_species_patch: #(
		#(500 500 500 500 500)
		#(4990 5000 5000 5000 5000)
	);
	I_species_patch: #(
		#(0 0 0 0 0)
		#(10 0 0 0 0)
	).
		
Simulation SpatialRungeKutta rungeKutta
	for: 'MultiSpeciesSpatial';
	from: 0; 
	to: 500; 
	step: 1.
	
Visualization SpatialDiagramViz diagram 
	for: 'SpatialRungeKutta';
	data: #(I_species);
	legendTitle: 'Total of Infectious';
	legends: #('humans' 'birds');
	xLabel: 'Time (days)'.! !

!TestMultiSpeciesSpatialDependant methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 21:50'!
afterBench

KendrickModel Infuenza.
	
Concern Biological
	attribute: #(species -> human bird).
	
Concern Demographical
	attribute: #(patch -> 1  2  3  4  5);
	parameters: #(rho);
	rho: 0.03;
	transitions: #(
		1 -- rho --> 2.
		2 -- rho --> 3.
		3 -- rho --> 4.
		4 -- rho --> 5.
		5 -- rho --> 1.
	).
		
Concern SIR
	attribute: #(status -> S I R);
	parameters: #(beta lambda gamma mu v);
	gamma: 0.25;
	v: 0.00274;
	transitions: #(
		S -- lambda --> I.
		I -- gamma --> R.
		R -- v --> S.
	).	
		
Concern SEIR
	extends: 'SIR';
	parameters: #(sigma);
	addTransition:	#(Empty -- mu --> S.);
	addTransition: #(status -- mu --> Empty.);
	delay: #(sigma , S -- lambda --> I  , E).
		
Composition MultiSpeciesSpatial
	model: 'Infuenza';
	populationSize: 27500;
	concern: 'Demographical';
	concern: 'Biological';
	concern: 'SEIR';
	mu_species: #(0.0000365 0.00137);
	sigma_species: #(0.5 0.67);
	gamma_species: #(0.25 0.233);
	rho_species: #(0.03 0.1);
	beta_species: #(
		#(0 0.21)
		#(0 0.42)
	);
	lambda: #(beta*I_patch/N sum);
	N: #(patch_species);
	S_species_patch: #(
		#(500 500 500 500 500)
		#(4990 5000 5000 5000 5000)
	);
	I_species_patch: #(
		#(0 0 0 0 0)
		#(10 0 0 0 0)
	).
		
Simulation SpatialRungeKutta rungeKutta
	for: 'MultiSpeciesSpatial';
	from: 0; 
	to: 500; 
	step: 1;
	init;
	execute.! !

!TestMultiSpeciesSpatialDependant methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 18:50'!
before

|model sirConcern eConcern spatialConcern simulator db multiHostConcern seirsConcern f |
model := KEModel new population: (KEPopulation size: 27500).

"Multi-host concern"
multiHostConcern := KEConcern new.
multiHostConcern
   addAttribute: #species 
   value: #(#human #bird).

"Spatial concern"
spatialConcern := KEConcern new.
spatialConcern addAttribute: #patch value: (1 to: 5) asArray.
spatialConcern addParameter: #rho.
    (1 to: 5) do: [ :i| 
        (i < 5) 
        ifTrue: [  
            spatialConcern 
                addTransitionFrom: { #patch->i } 
                to: { #patch->(i+1) } 
                probability: [ :aModel| aModel atParameter: #rho ].
        ]
        ifFalse: [  
            spatialConcern 
                addTransitionFrom: { #patch->i } 
                to: { #patch->1 } 
                probability: [ :aModel| aModel atParameter: #rho ].
        ]
    ]. 

"SEIRS concern"
seirsConcern := KEConcern new.
seirsConcern attributes: {#status->#(#S #E #I #R)}.
seirsConcern addParameters: { #beta. #gamma. #mu. #sigma. #nu }.
seirsConcern addParameter: #lambda value: [ :aModel | 
            (aModel atParameter: #beta) * 
            (aModel atCompartment: {#status->#I}) / (aModel atParameter: #N) ].
seirsConcern
        addTransitionFrom: '{#status: #S}'
        to: '{#status: #E}'
        probability: [ :aModel | 
            (aModel atParameter: #lambda) ].
    seirsConcern 
        addTransitionFrom: '{#status: #E}' 
        to: '{#status: #I}' 
        probability: [ :aModel | aModel atParameter: #sigma ].
    seirsConcern 
        addTransitionFrom: '{#status: #I}' 
        to: '{#status: #R}' 
        probability: [ :aModel | aModel atParameter: #gamma ].
    seirsConcern 
        addTransitionFrom: '{#status: #R}' 
        to: '{#status: #S}' 
        probability: [ :aModel | aModel atParameter: #nu ].
    #(S E I R) do: [ :each| 
        seirsConcern 
            addTransitionFrom: {#status->each}
            to: #empty 
            probability: [ :aModel | aModel atParameter: #mu ].
        ].
    seirsConcern 
        addTransitionFrom: #empty 
        to: '{#status: #S}' 
        probability: [ :aModel | aModel atParameter: #mu ].

"Concerns integration"

    model integrate: spatialConcern.
    model integrate: multiHostConcern.
    model integrate: seirsConcern.

    model atParameter: #mu assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #species.
        c = #human ifTrue: [ val := 0.0000365 ].
        c = #bird ifTrue: [ val := 0.00137 ].
        val ].
    model atParameter: #sigma assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #species.
        c = #human ifTrue: [ val := 0.5 ].
        c = #bird ifTrue: [ val := 0.67 ].
        val ].
    model atParameter: #gamma assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #species.
        c = #human ifTrue: [ val := 0.25 ].
        c = #bird ifTrue: [ val := 0.233 ].
        val ].
    model atParameter: #nu assignValue: 0.00274.
    model atParameter: #N assignValue: [ :aModel| |c| 
        c := OrderedCollection new.
        c add: (aModel currentCompartment at: #patch).
        c add: (aModel currentCompartment at: #species).
        aModel sizeOfPopulation: (c asArray)
    ].
    model atParameter: #rho assignValue: [ :aModel| |c val|
        c := aModel currentCompartment at: #species.
        c = #bird ifTrue: [ val := 0.1 ].
        c = #human ifTrue: [ val := 0.03 ].
        val
    ].
    model atParameter: #beta assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #species.
        c = #human ifTrue: [ val := #(0 0.21) ].
        c = #bird ifTrue: [ val := #(0 0.42) ].
        val
    ]. 
    model atParameter: #lambda assignValue: [ :aModel| |c|
        c := aModel currentCompartment at: #patch.
        ((aModel atParameter: #beta) *
        (aModel atCompartment: {#status->#I. #patch->c}) / (aModel atParameter: #N)) sum.
    ].
    model 
        atCompartment: { #status->#S. #species->#bird. #patch->1 } 
        put: 4990
        atOthersPut: 0.
    model 
        atCompartment: { #status->#I. #species->#bird. #patch->1 } 
        put: 10.
    2 to: 5 do: [ :i| 
        model 
            atCompartment: { #status->#S. #species->#bird. #patch->i } 
            put: 5000.
    ].
    1 to: 5 do: [ :i| 
        model 
            atCompartment: { #status->#S. #species->#human. #patch->i } 
            put: 500 
    ].

"Run the model with RK4"
simulator := KESimulator new: #RungeKutta from: 0 to: 500 step: 1.
    simulator executeOn: model.
    db := KEDiagramBuilder new.

    f := [:name| |d tmp|
        tmp := (simulator timeSeriesAt: name) collect: [ :e| e value ].
        d := OrderedCollection new.
        1 to: tmp first data size do: [ :k| d add: (tmp collect: [:e| e data at: k]) sum ].
        (KETimeSeries from: d withIndex: tmp first index) compartment: (STON fromString: name)
        ].
    db data: { 
        (f value: '{#status:#I, #species:#bird}'). 
        (f value: '{#status: #I, #species: #human}') }.
    db xLabel: 'Time (days)'.
    db legendTitle: 'Total of infectious'.
    db legends: { 'birds'. 'humans' }.

    db open.

	^ { simulator . model . db}! !

!TestMultiSpeciesSpatialDependant methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 21:51'!
beforeBench

|model sirConcern eConcern spatialConcern simulator db multiHostConcern seirsConcern f |
model := KEModel new population: (KEPopulation size: 27500).

"Multi-host concern"
multiHostConcern := KEConcern new.
multiHostConcern
   addAttribute: #species 
   value: #(#human #bird).

"Spatial concern"
spatialConcern := KEConcern new.
spatialConcern addAttribute: #patch value: (1 to: 5) asArray.
spatialConcern addParameter: #rho.
    (1 to: 5) do: [ :i| 
        (i < 5) 
        ifTrue: [  
            spatialConcern 
                addTransitionFrom: { #patch->i } 
                to: { #patch->(i+1) } 
                probability: [ :aModel| aModel atParameter: #rho ].
        ]
        ifFalse: [  
            spatialConcern 
                addTransitionFrom: { #patch->i } 
                to: { #patch->1 } 
                probability: [ :aModel| aModel atParameter: #rho ].
        ]
    ]. 

"SEIRS concern"
seirsConcern := KEConcern new.
seirsConcern attributes: {#status->#(#S #E #I #R)}.
seirsConcern addParameters: { #beta. #gamma. #mu. #sigma. #nu }.
seirsConcern addParameter: #lambda value: [ :aModel | 
            (aModel atParameter: #beta) * 
            (aModel atCompartment: {#status->#I}) / (aModel atParameter: #N) ].
seirsConcern
        addTransitionFrom: '{#status: #S}'
        to: '{#status: #E}'
        probability: [ :aModel | 
            (aModel atParameter: #lambda) ].
    seirsConcern 
        addTransitionFrom: '{#status: #E}' 
        to: '{#status: #I}' 
        probability: [ :aModel | aModel atParameter: #sigma ].
    seirsConcern 
        addTransitionFrom: '{#status: #I}' 
        to: '{#status: #R}' 
        probability: [ :aModel | aModel atParameter: #gamma ].
    seirsConcern 
        addTransitionFrom: '{#status: #R}' 
        to: '{#status: #S}' 
        probability: [ :aModel | aModel atParameter: #nu ].
    #(S E I R) do: [ :each| 
        seirsConcern 
            addTransitionFrom: {#status->each}
            to: #empty 
            probability: [ :aModel | aModel atParameter: #mu ].
        ].
    seirsConcern 
        addTransitionFrom: #empty 
        to: '{#status: #S}' 
        probability: [ :aModel | aModel atParameter: #mu ].

"Concerns integration"

    model integrate: spatialConcern.
    model integrate: multiHostConcern.
    model integrate: seirsConcern.

    model atParameter: #mu assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #species.
        c = #human ifTrue: [ val := 0.0000365 ].
        c = #bird ifTrue: [ val := 0.00137 ].
        val ].
    model atParameter: #sigma assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #species.
        c = #human ifTrue: [ val := 0.5 ].
        c = #bird ifTrue: [ val := 0.67 ].
        val ].
    model atParameter: #gamma assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #species.
        c = #human ifTrue: [ val := 0.25 ].
        c = #bird ifTrue: [ val := 0.233 ].
        val ].
    model atParameter: #nu assignValue: 0.00274.
    model atParameter: #N assignValue: [ :aModel| |c| 
        c := OrderedCollection new.
        c add: (aModel currentCompartment at: #patch).
        c add: (aModel currentCompartment at: #species).
        aModel sizeOfPopulation: (c asArray)
    ].
    model atParameter: #rho assignValue: [ :aModel| |c val|
        c := aModel currentCompartment at: #species.
        c = #bird ifTrue: [ val := 0.1 ].
        c = #human ifTrue: [ val := 0.03 ].
        val
    ].
    model atParameter: #beta assignValue: [ :aModel| |c val| 
        c := aModel currentCompartment at: #species.
        c = #human ifTrue: [ val := #(0 0.21) ].
        c = #bird ifTrue: [ val := #(0 0.42) ].
        val
    ]. 
    model atParameter: #lambda assignValue: [ :aModel| |c|
        c := aModel currentCompartment at: #patch.
        ((aModel atParameter: #beta) *
        (aModel atCompartment: {#status->#I. #patch->c}) / (aModel atParameter: #N)) sum.
    ].
    model 
        atCompartment: { #status->#S. #species->#bird. #patch->1 } 
        put: 4990
        atOthersPut: 0.
    model 
        atCompartment: { #status->#I. #species->#bird. #patch->1 } 
        put: 10.
    2 to: 5 do: [ :i| 
        model 
            atCompartment: { #status->#S. #species->#bird. #patch->i } 
            put: 5000.
    ].
    1 to: 5 do: [ :i| 
        model 
            atCompartment: { #status->#S. #species->#human. #patch->i } 
            put: 500 
    ].

"Run the model with RK4"
simulator := KESimulator new: #RungeKutta from: 0 to: 500 step: 1.
    simulator executeOn: model.
! !

!TestMultiSpeciesSpatialDependant methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/20/2016 13:32'!
bench

	| before after |
	
	before := self benchmark: [self beforeBench] times: 100.
	after := self benchmark: [self afterBench] times: 100.
	
	Transcript 
		show: before; cr; 
		show: after; cr; 
		show: (after at: 1) / (before at: 1); cr; cr.! !

!TestMultiSpeciesSpatialDependant methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 21:16'!
benchmark: aBlock times: anInteger

	"µ the average time in ms and τ standard deviation"

	| runs average deviation |

	runs := [ 
		(1 to: anInteger) 
			collect: [ :each | Time millisecondsToRun: aBlock ] ] 
		valueUnpreemptively.
	average := runs average asFloat.
	deviation := ((runs 
		detectSum: [ :each | (each - average) * (each - average) ]) 
			/ runs size) sqrt.
	^ Array with: average with: deviation! !

!TestMultiSpeciesSpatialDependant methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 18:53'!
createModels

	| beforeList |
	beforeList := self before.
	beforeSimulator := beforeList first.
	beforeModel := beforeList second.
	beforeDiagram := beforeList third.
	self after.
	afterSimulator := Simulation withId: 'SpatialRungeKutta'.
	afterModel := KendrickModel withId: 'Infuenza'.
	afterDiagram := (Visualization withId: 'SpatialDiagramViz') open.	! !

!TestMultiSpeciesSpatialDependant methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/17/2016 16:42'!
setUp

	KendrickInternalDSLEntities clearAll.
	"KEGillespieSimulatorForTesting newSeed." "stochastic but reproducible each time"! !

!TestMultiSpeciesSpatialDependant methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/17/2016 16:42'!
tearDown

	KendrickInternalDSLEntities clearAll.! !

!TestMultiSpeciesSpatialDependant methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 20:43'!
testEquivalence


	| resultsBefore resultsAfter |
	self createModels.

	beforeModel inspect.
	afterModel keModel inspect.

	"correct (check inspector) -- but timeseries are presented in different order fix"

	resultsBefore := (beforeDiagram data collect: [ :ts | ts peakOfEpidemic floor ]) sorted asOrderedCollection.
	resultsAfter := (afterDiagram data collect: [ :ts | ts peakOfEpidemic floor ]) sorted asOrderedCollection.
	Transcript show: resultsBefore; cr; show: resultsAfter; cr; cr.
	self assert: resultsBefore = resultsAfter.! !

TestCase subclass: #TestMultiSpeciesSpatialODEs
	instanceVariableNames: 'beforeSimulator afterSimulator beforeModel afterModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestMultiSpeciesSpatialODEs methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/17/2016 18:51'!
after

KendrickModel MultiSpeciesSpatial	
	population: 27500;
	attributes: #(status -> S E I R);
	parameters: #(N mu sigma gamma rho beta p q s i);
	S: #(
			#(500 4990)
			#(500 5000)
			#(500 5000)
			#(500 5000)
			#(500 5000)
	);
	I: #(
		#(0 10)
		#(0 0)
		#(0 0)
		#(0 0)
		#(0 0)
	);
	N: #(
		#(500 5000)
		#(500 5000)
		#(500 5000)
		#(500 5000)
		#(500 5000)
	);
	mu: #(
		#(0.000365 0.00137)
		#(0.000365 0.00137)
		#(0.000365 0.00137)
		#(0.000365 0.00137)
		#(0.000365 0.00137)
	);
	sigma: #(
		#(0.5 0.67)
		#(0.5 0.67)
		#(0.5 0.67)
		#(0.5 0.67)
		#(0.5 0.67)
	);
	gamma: #(
		#(0.25 0.233)
		#(0.25 0.233)
		#(0.25 0.233)
		#(0.25 0.233)
		#(0.25 0.233)
	);
	rho: #(
		#(#(0.0 0.03 0.0 0.0 0.0) #(0.0 0.0 0.03 0.0 0.0) #(0.0 0.0 0.0 0.03 0.0) #(0.0 0.0 0.0 0.0 0.03) #(0.03 0.0 0.0 0.0 0.0))
		#(#(0.0 0.1 0.0 0.0 0.0) #(0.0 0.0 0.1 0.0 0.0) #(0.0 0.0 0.0 0.1 0.0) #(0.0 0.0 0.0 0.0 0.1) #(0.1 0.0 0.0 0.0 0.0))
	);
	beta: #(#(0 0.21) #(0 0.42));
	p: #(1 2 3 4 5);
	q: #(1 2 3 4 5);
	s: #(1 2);
	i: #(1 2);
	equations: #(
		S[p][s]:t=mu[p][s]*N[p][s]-sum(i,beta[i][s]*I[p][i])/N[p][i]*S[p][s]-mu[p][s]*S[p][s]+sum(q,rho[s][p][q]*S[q][s])-sum(q,rho[s][q][p])*S[p][s]+nu*R[p][s].
		E[p][s]:t=sum(i,beta[i][s]*I[p][i])/N[p][i]*S[p][s]-sigma[p][s]*E[p][s]-mu[p][s]*E[p][s]+sum(q,rho[s][p][q]*E[q][s])-sum(q,rho[s][q][p])*E[p][s].
		I[p][s]:t=sigma[p][s]*E[p][s]-gamma[p][s]*I[p][s]-mu[p][s]*I[p][s]+sum(q,rho[s][p][q]*I[q][s])-sum(q,rho[s][q][p])*I[p][s].
		R[p][s]:t=gamma[p][s]*I[p][s]-mu[p][s]*I[p][s]-nu*R[p][s]+sum(q,rho[s][p][q]*R[q][s])-sum(q,rho[s][q][p])*R[p][s].
	).
		
Simulation SpatialRungeKutta rungeKutta
	for: 'MultiSpeciesSpatial';
	from: 0; 
	to: 500; 
	step: 1.
	
Visualization SpatialDiagramViz diagram 
	forModel: 'SpatialRungeKutta';
	data: #(I_species);"seems to be need some more handling here"
	legendTitle: 'Total of Infectious';
	legends: #( 'humans' 'birds' );
	xLabel: 'Time (days)'.! !

!TestMultiSpeciesSpatialODEs methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/25/2016 15:24'!
before

|model simulator db f |
model := KEModel new.
model
	atCompartment: #S
	put: #(#(500 4990)#(500 5000)#(500 5000)#(500 5000)#(500 5000))
	atOthersPut: #(#(0 0)#(0 0)#(0 0)#(0 0)#(0 0)).
model
	atCompartment: #I
	put: #(#(0 10)#(0 0)#(0 0)#(0 0)#(0 0)).
model
	atParameter: #N
	assignValue: #(#(500 5000)#(500 5000)#(500 5000)#(500 5000)#(500 5000)).
model
	atParameter: #mu
	assignValue: #(#(0.000365 0.00137)#(0.000365 0.00137)#(0.000365 0.00137)#(0.000365 0.00137)#(0.000365 0.00137)).
model
	atParameter: #sigma
	assignValue: #(#(0.5 0.67)#(0.5 0.67)#(0.5 0.67)#(0.5 0.67)#(0.5 0.67)).

model
	atParameter: #gamma
	assignValue: #(#(0.25 0.233)#(0.25 0.233)#(0.25 0.233)#(0.25 0.233)#(0.25 0.233)).

model
	atParameter: #nu assignValue: 0.00274.

model
	atParameter: #rho
	assignValue: {0.03*#(#(0 1 0 0 0)#(0 0 1 0 0)#(0 0 0 1 0)#(0 0 0 0 1)#(1 0 0 0 0))
		. 0.1*#(#(0 1 0 0 0)#(0 0 1 0 0)#(0 0 0 1 0)#(0 0 0 0 1)#(1 0 0 0 0))}.
model
	atParameter: #beta assignValue: #(#(0 0.21) #(0 0.42)).

model
	atParameter: #p assignValue: (1 to: 5).

model
	atParameter: #q assignValue: (1 to: 5).

model
	atParameter: #s assignValue: (1 to: 2).

model
	atParameter: #i assignValue: (1 to: 2).

model addEquation:
	'S[p][s]:t=mu[p][s]*N[p][s]-sum(i,beta[i][s]*I[p][i])/N[p][i]*S[p][s]-mu[p][s]*S[p][s]+sum(q,rho[s][p][q]*S[q][s])-sum(q,rho[s][q][p])*S[p][s]+nu*R[p][s]' parseAsAnEquation.
model addEquation:
	'E[p][s]:t=sum(i,beta[i][s]*I[p][i])/N[p][i]*S[p][s]-sigma[p][s]*E[p][s]-mu[p][s]*E[p][s]+sum(q,rho[s][p][q]*E[q][s])-sum(q,rho[s][q][p])*E[p][s]' parseAsAnEquation.
model addEquation:
	'I[p][s]:t=sigma[p][s]*E[p][s]-gamma[p][s]*I[p][s]-mu[p][s]*I[p][s]+sum(q,rho[s][p][q]*I[q][s])-sum(q,rho[s][q][p])*I[p][s]' parseAsAnEquation.
model addEquation:
	'R[p][s]:t=gamma[p][s]*I[p][s]-mu[p][s]*I[p][s]-nu*R[p][s]+sum(q,rho[s][p][q]*R[q][s])-sum(q,rho[s][q][p])*R[p][s]' parseAsAnEquation.

"Run the model with RK4"
simulator := KESimulator new: #RungeKutta from: 0 to: 500 step: 1.
    simulator executeOn: model.
    db := KEDiagramBuilder new.

    f := [:name| |d tmp|
        tmp := (simulator timeSeriesAt: name) collect: [ :e| e value ].
        d := OrderedCollection new.
        1 to: tmp first data size do: [ :k| d add: (tmp collect: [:e| e data at: k]) sum ].
        (KETimeSeries from: d withIndex: tmp first index) compartment: (STON fromString: name)
        ].
    db data: { 
        (f value: '{#status:#I, #species:#bird}'). 
        (f value: '{#status: #I, #species: #human}') }.
    db xLabel: 'Time (days)'.
    db legendTitle: 'Total of infectious'.
    db legends: { 'birds'. 'humans' }.

    db open.

	^ simulator -> model! !

!TestMultiSpeciesSpatialODEs methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/25/2016 15:14'!
createModels

	| beforePair |
	beforePair := self before.
	beforeSimulator := beforePair key.
	beforeModel := beforePair value.
	self after.
	afterSimulator := Simulation withId: 'SpatialRungeKutta'.
	afterModel := KendrickModel withId: 'MultiSpeciesSpatial'.
	(Visualization withId: 'SpatialDiagramViz') open.	! !

!TestMultiSpeciesSpatialODEs methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/25/2016 15:14'!
setUp

	KendrickInternalDSLEntities clearAll.
	"KEGillespieSimulatorForTesting newSeed." "stochastic but reproducible each time"! !

!TestMultiSpeciesSpatialODEs methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/25/2016 15:14'!
tearDown

	KendrickInternalDSLEntities clearAll.! !

!TestMultiSpeciesSpatialODEs methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 20:44'!
testEquivalence! !

TestCase subclass: #TestOneSpeciesSpatialScripts
	instanceVariableNames: 'beforeSimulator afterSimulator beforeModel afterModel beforeDiagram afterDiagram'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestOneSpeciesSpatialScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/17/2016 18:44'!
after

KendrickModel OneSpeciesSpatial	
	population: 5000.

Concern Spatial
	attribute: #(patch -> 1  2  3  4  5);
	parameters: #(rho);
	rho: 0.03;
	transitions: #(
		1 -- rho --> 2.
		2 -- rho --> 3.
		3 -- rho --> 4.
		4 -- rho --> 5.
		5 -- rho --> 1.
	).
		
Concern Epidemiological
	attribute: #(status -> S  E  I  R);
	parameters: #(beta lambda sigma gamma v mu);
	v: 0.00274;
	mu: 0.0000365;
	sigma: 0.5;
	gamma: 0.25;
	lambda: #(beta*I/N);
	transitions: #(
		S -- lambda --> E.
		E -- sigma --> I.
		I -- gamma --> R.
		R -- v --> S.
		status -- mu --> Empty.
		Empty -- mu --> S.
	).
		
Composition OneSpatialCombo
	model: 'OneSpeciesSpatial';
	concern: 'Spatial';
	concern: 'Epidemiological';
	beta_patch: #(0.75 0.5 0.5 0.5 0.5);
	N: #(patch);
	lambda: #(beta*I_patch/N); "check how this should work"
	S_patch: #(900 1000 1000 1000 1000);
	I_patch: #(100 0 0 0 0).
		
Simulation SpatialRungeKutta rungeKutta
	for: 'OneSpatialCombo';
	from: 0; 
	to: 500; 
	step: 1.
	
Visualization SpatialDiagramViz diagram 
	for: 'SpatialRungeKutta';
	data: #(I_patch);
	legends: #('patch 1' 'patch 2' 'patch 3' 'patch 4' 'patch 5');
	legendTitle: 'Infectious';
	xLabel: 'Time (days)'.! !

!TestOneSpeciesSpatialScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 20:47'!
before

| spatialConcern seirsConcern model simulator db |
spatialConcern := KEConcern new.
 spatialConcern addAttribute: #patch value: (1 to: 5) asArray.
 spatialConcern addParameter: #rho.
 (1 to: 5) do: [ :i| 
     (i < 5) 
     ifTrue: [  
         spatialConcern 
             addTransitionFrom: { #patch->i } 
             to: { #patch->(i+1) } 
             probability: [ :aModel| aModel atParameter: #rho ].
     ]
     ifFalse: [  
         spatialConcern 
             addTransitionFrom: { #patch->i } 
             to: { #patch->1 } 
             probability: [ :aModel| aModel atParameter: #rho ].
     ]
 ]. 

seirsConcern := KEConcern new.
 seirsConcern attributes: {#status->#(#S #E #I #R)}.
 seirsConcern addParameters: { #beta. #gamma. #mu. #sigma. #v }.
 seirsConcern addParameter: #lambda value: [ :aModel | 
         (aModel atParameter: #beta) * 
         (aModel atCompartment: {#status->#I}) / (aModel atParameter: #N) ].
 seirsConcern
     addTransitionFrom: '{#status: #S}'
     to: '{#status: #E}'
     probability: [ :aModel | 
         (aModel atParameter: #lambda) ].
 seirsConcern 
     addTransitionFrom: '{#status: #E}' 
     to: '{#status: #I}' 
     probability: [ :aModel | aModel atParameter: #sigma ].
 seirsConcern 
     addTransitionFrom: '{#status: #I}' 
     to: '{#status: #R}' 
     probability: [ :aModel | aModel atParameter: #gamma ].
 seirsConcern 
     addTransitionFrom: '{#status: #R}' 
     to: '{#status: #S}' 
     probability: [ :aModel | aModel atParameter: #v ].
 #(S E I R) do: [ :each| 
     seirsConcern 
         addTransitionFrom: {#status->each}
         to: #empty 
         probability: [ :aModel | aModel atParameter: #mu ].
     ].
 seirsConcern 
     addTransitionFrom: #empty 
     to: '{#status: #S}' 
     probability: [ :aModel | aModel atParameter: #mu ].

     model := KEModel new.
 model population: (KEPopulation size: 5000).
 model integrate: spatialConcern.
 model integrate: seirsConcern.

model atParameter: #beta assignValue: [ :aModel| |c val| 
     c := aModel currentCompartment at: #patch.
     c = 1 ifTrue: [ val := 0.75 ] ifFalse: [ val := 0.5 ].
     val 
 ].
 model atParameter: #v assignValue: 0.00274.
 model atParameter: #mu assignValue: 0.0000365.
 model atParameter: #sigma assignValue: 0.5.
 model atParameter: #gamma assignValue: 0.25.
 model atParameter: #N assignValue: [ :aModel| |c| 
     c := aModel currentCompartment at: #patch.
     aModel sizeOfPopulation: c
     ].
 model atParameter: #rho assignValue: 0.03. 
 model atParameter: #lambda assignValue: [ :aModel| |c|
     c := aModel currentCompartment at: #patch.
     (aModel atParameter: #beta) * (aModel atCompartment: {#status->#I. #patch->c}) / (aModel atParameter: #N)
 ].

model atCompartment: { #status->#S. #patch->1 } put: 900 atOthersPut: 0.
 model atCompartment: { #status->#I. #patch->1 } put: 100.
 model atCompartment: { #status->#S. #patch->2 } put: 1000.
 model atCompartment: { #status->#S. #patch->3 } put: 1000.
 model atCompartment: { #status->#S. #patch->4 } put: 1000.
 model atCompartment: { #status->#S. #patch->5 } put: 1000.

simulator := KESimulator new: #RungeKutta from: 0 to: 500 step: 1.
 simulator executeOn: model.
 db := KEDiagramBuilder new.
 db data: [|tmp| tmp := OrderedCollection new.
 tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:1}').
 tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:2}').
 tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:3}').
 tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:4}').
 tmp addAll: (simulator timeSeriesAt: '{#status: #I,#patch:5}').
 tmp] value.
 db legends: { 'patch 1'. 'patch 2'. 'patch 3'. 'patch 4'. 'patch 5'}.
 db legendTitle: 'Infectious'.
 db xLabel: 'Time (days)'.
 db open.

	^ {simulator . model . db}! !

!TestOneSpeciesSpatialScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 20:56'!
createModels

	| beforeList |
	beforeList := self before.
	beforeSimulator := beforeList first.
	beforeModel := beforeList second.
	beforeDiagram := beforeList third.
	self after.
	afterSimulator := Simulation withId: 'SpatialRungeKutta'.
	afterModel := KendrickModel withId: 'OneSpeciesSpatial'.
	afterDiagram := (Visualization withId: 'SpatialDiagramViz') open.	! !

!TestOneSpeciesSpatialScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/19/2016 17:18'!
setUp

	KendrickInternalDSLEntities clearAll.
	"KEGillespieSimulatorForTesting newSeed." "stochastic but reproducible each time"! !

!TestOneSpeciesSpatialScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/6/2016 16:58'!
tearDown

	KendrickInternalDSLEntities clearAll.! !

!TestOneSpeciesSpatialScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 20:56'!
testEquivalence


	| resultsBefore resultsAfter |
	self createModels.

	beforeModel inspect.
	afterModel keModel inspect.

	"correct (check inspector) -- but timeseries are presented in different order fix"

	resultsBefore := (beforeDiagram data collect: [ :ts | ts peakOfEpidemic floor ]) sorted asOrderedCollection.
	resultsAfter := (afterDiagram data collect: [ :ts | ts peakOfEpidemic floor ]) sorted asOrderedCollection.
	Transcript show: resultsBefore; cr; show: resultsAfter; cr; cr.
	self assert: resultsBefore = resultsAfter.! !

TestCase subclass: #TestsIST
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestsIST methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/27/2016 11:50'!
script! !

!TestsIST methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/27/2016 11:58'!
setUp

	KendrickInternalDSLEntities clearAll.! !

!TestsIST methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/11/2016 18:08'!
tearDown

	KendrickInternalDSLEntities clearAll.! !

!TestsIST methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/27/2016 17:28'!
testScript

	"need smth stronger here like the before/after tests on TestsMultiSpecies etc"
	self script.
	(Visualization withId: self vizId) open.! !

!TestsIST methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/27/2016 11:54'!
vizId

	self subclassResponsibility ! !

TestsIST subclass: #TestsModel1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestsModel1 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/11/2016 14:34'!
script

KendrickModel Influenza.
	
Concern SIR
	attribute: #(status -> S I R);
	parameters: #(beta lambda gamma mu);
	lambda: #(beta*I/N);
	transitions: #(
		S -- lambda --> I.
		I -- gamma --> R.
		status -- mu --> Empty.
		Empty -- mu --> S.
	).

Composition SIRInfluenza
	model: 'Influenza';
	concern: 'SIR';
	populationSize: 5000;
	S: 4990;
	I: 10;
	others: 0;
	mu: 0.00137;
	gamma: 0.233;
	beta: 0.42.
	
Simulation SIRInfluenzaRK rungeKutta
	for: 'SIRInfluenza';
	from: 0; 
	to: 500; 
	step: 1.
	
Visualization SIRInfluenzaViz diagram 
	for: 'SIRInfluenzaRK';
	xLabel: 'Time (days)'.! !

!TestsModel1 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/27/2016 12:07'!
vizId

	^ 'SIRInfluenzaViz'! !

TestsIST subclass: #TestsModel2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestsModel2 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/11/2016 14:39'!
script

KendrickModel Influenza.
	
Concern SIR
	attribute: #(status -> S I R);
	parameters: #(beta lambda gamma mu);
	lambda: #(beta*I/N);
	transitions: #(
		S -- lambda --> I.
		I -- gamma --> R.
		status -- mu --> Empty.
		Empty -- mu --> S.).

Concern Biological
	attribute: #(species -> human bird).
	
Composition SIRInfluenza
	model: 'Influenza';
	populationSize: 5500;
	concern: 'Biological';
	concern: 'SIR';
	mu_species: #(0.0000365 0.00137);
	gamma_species: #(0.25 0.233);
	beta_species: #(#(0 0.21) #(0 0.42));
	lambda: #(beta*I/N sum);
	N: #(species);
	S_species: #(500 4990);
	I_species: #(0 10).
	
Simulation SIRInfluenzaRK rungeKutta
	for: 'SIRInfluenza';
	from: 0; 
	to: 500; 
	step: 1.
	
Visualization SIRInfluenzaViz diagram 
	for: 'SIRInfluenzaRK';
	xLabel: 'Time (days)'.! !

!TestsModel2 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/11/2016 14:41'!
vizId

	^ 'SIRInfluenzaViz'! !

TestsIST subclass: #TestsModel3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestsModel3 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/11/2016 14:43'!
script

KendrickModel Influenza.

Concern SIR
	attribute: #(status -> S I R);
	parameters: #(beta lambda gamma mu);
	lambda: #(beta*I/N);
	transitions: #(
		S -- lambda --> I.
		I -- gamma --> R.
		status -- mu --> Empty.
		Empty -- mu --> S.).

Concern Biological
	attribute: #(species -> human bird).
	
Concern Demographical
	attribute: #(patch -> 1  2  3  4  5);
	parameters: #(rho);
	rho: 0.03;
	transitions: #(
		1 -- rho --> 2.
		2 -- rho --> 3.
		3 -- rho --> 4.
		4 -- rho --> 5.
		5 -- rho --> 1.).
		
Concern SEIRS
	extends: 'SIR';
	parameters: #(sigma nu);
	delay: #(sigma , S -- lambda --> I  , E);
	addTransition:	#(R -- nu --> S.);
	addTransition: #(E -- mu --> Empty.).
		
Composition SEIRSInfluenza
	model: 'Influenza';
	populationSize: 27500;
	concern: 'Demographical';
	concern: 'Biological';
	concern: 'SEIRS';
	nu: 0.00274;
	mu_species: #(0.0000365 0.00137);
	sigma_species: #(0.5 0.67);
	gamma_species: #(0.25 0.233);
	rho_species: #(0.03 0.1);
	beta_species: #(#(0 0.21) #(0 0.42));
	lambda: #(beta*I_patch/N sum);
	N: #(patch_species);
	S_species_patch: #(#(500 500 500 500 500) #(4990 5000 5000 5000 5000));
	I_species_patch: #(#(0 0 0 0 0) #(10 0 0 0 0)).
		
Simulation SEIRSInfluenzaRK rungeKutta
	for: 'SEIRSInfluenza';
	from: 0; 
	to: 500; 
	step: 1.
	
Visualization SEIRSInfluenzaViz diagram 
	for: 'SEIRSInfluenzaRK';
	data: #(I_species);
	legendTitle: 'Total of Infectious';
	legends: #('humans' 'birds');
	xLabel: 'Time (days)'.! !

!TestsModel3 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/11/2016 14:44'!
vizId

	^ 'SEIRSInfluenzaViz'! !

TestsIST subclass: #TestsModel4
	instanceVariableNames: 'beforeSimulator beforeModel beforeDiagram afterSimulator afterModel afterDiagram'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestsModel4 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/27/2016 17:35'!
script

KendrickModel Influenza.

Concern SIR
	attribute: #(status -> S I R);
	parameters: #(beta lambda gamma mu);
	lambda: #(beta*I/N);
	transitions: #(
		S -- lambda --> I.
		I -- gamma --> R.
		status -- mu --> Empty.
		Empty -- mu --> S.
	).	

Concern Biological
	attribute: #(species -> human bird).
	
Concern Demographical
	attribute: #(patch -> 1  2  3  4  5);
	parameters: #(rho);
	rho: 0.03;
	transitions: #(
		1 -- rho --> 2.
		2 -- rho --> 3.
		3 -- rho --> 4.
		4 -- rho --> 5.
		5 -- rho --> 1.).
		
Concern SEIRS
	extends: 'SIR';
	parameters: #(sigma nu);
	delay: #(sigma , S -- lambda --> I  , E);
	addTransition:	#(R -- nu --> S.);
	addTransition: #(E -- mu --> Empty.).

Concern SEIRSQ
	extends: 'SEIRS';
	parameters: #(delta epsilon);
	addStatus: #(Q);
	addTransition: #(I -- delta --> Q.);
	addTransition: #(Q -- epsilon --> R.);
	addTransition: #(Q -- mu --> Empty.);
	lambda: #(beta*I/(N-Q)).
	
Composition SEIRSQInfluenza
	model: 'Influenza';
	populationSize: 27500;
	concern: 'SEIRSQ';
	concern: 'Demographical';
	concern: 'Biological';
	beta_species: #(#(0 0.21) #(0 0.42));
	lambda: #(beta*(I_patch/(N-Q_patch)) sum);
	gamma_species: #(0.25 0.233);
	sigma_species: #(0.5 0.67);
	mu_species: #(0.0000365 0.00137);
	N: #(species_patch);
	nu: 0.00274;		
	delta_species: #(0.068 0.055);
	epsilon_species: #(0.096 0.082);
	rho_species_Q: #(0.03 0.1 0);
	S_species_patch: #(#(500 500 500 500 500) #(4990 5000 5000 5000 5000));
	I_species_patch: #(#(0 0 0 0 0) #(10 0 0 0 0)).

Simulation SEIRSQInfluenzaRK rungeKutta
	for: 'SEIRSQInfluenza';
	from: 0; 
	to: 500; 
	step: 1.
	
Visualization SEIRSQInfluenzaViz diagram 
	for: 'SEIRSQInfluenzaRK';
	data: #(I_species);
	legendTitle: 'Total of Infectious';
	legends: #('humans' 'birds');
	xLabel: 'Time (days)'.! !

!TestsModel4 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/11/2016 14:58'!
vizId

	^ 'SEIRSQInfluenzaViz'! !

TestsIST subclass: #TestsModel5
	instanceVariableNames: 'beforeSimulator beforeModel beforeDiagram afterSimulator afterModel afterDiagram'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!TestsModel5 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/28/2016 13:08'!
after

KendrickModel Influenza.

Concern SEIR
	attribute: #(status -> S E I R);
	parameters: #(beta lambda sigma gamma mu);
	lambda: #(beta*I/N);
	"transitions: #(
		S -- lambda --> I.
		I -- gamma --> R.
		status -- mu --> Empty.
		Empty -- mu --> S.
	).	"
	equations: #( 
		S:t=mu*N - lambda*S - mu*S.
		E:t=lambda*S - sigma*E - mu*E.
		I:t=sigma*E - gamma*I - mu*I.
		R:t=gamma*I - mu*R.
	).
		
Concern SEIRS
	extends: 'SEIR';
	parameters: #(nu);
	addTransition:	#(R -- nu --> S.).

Concern SEIIRS
	extends: 'SEIRS';
	parameters: #(beta1 beta2);
	splitStatus: #(I strains #(1 2)).

Concern Biological
	attribute: #(species -> human bird).
	
Concern Demographical
	attribute: #(patch -> 1  2  3  4  5);
	parameters: #(rho);
	rho: 0.03;
	transitions: #(
		1 -- rho --> 2.
		2 -- rho --> 3.
		3 -- rho --> 4.
		4 -- rho --> 5.
		5 -- rho --> 1.).
	
Composition SEIIRSCombo
	model: 'Influenza';
	populationSize: 27500;
	concern: 'SEIIRS';
	concern: 'Demographical';
	concern: 'Biological';
	beta1_species: #(#(0 0.21) #(0 0.42));
	beta2_species: #(#(0 0.021) #(0 0.042));
	lambda: #((beta1*(I1_patch/N))+(beta2*(I2_patch/N)) sum);
	gamma_species_status: #(
		#(0 0 0 0 0.25 0.0025)
		#(0 0 0 0 0.233 0.0023)
	);
	sigma_species_status_next: #(
		#(0 0 0 0 0.5 0.005)
		#(0 0 0 0 0.67 0.0067)
	);
	"gamma: 0.5;
	sigma: 0.6;"
	mu_species: #(0.000365 0.00137);
	N: #(patch_species);
	nu: 0.00274;		
	rho_species: #(0.03 0.1);
	S_species_patch: #(#(500 500 500 500 500) #(4990 5000 5000 5000 5000));
	I1_species_patch: #(#(0 0 0 0 0) #(9 0 0 0 0));
	I2_species_patch: #(#(0 0 0 0 0) #(1 0 0 0 0)).

Simulation SEIIRSInfluenzaRK rungeKutta
	for: 'SEIIRSCombo';
	from: 0.0; 
	to: 500; 
	step: 1.
	
Visualization SEIIRSInfluenzaViz diagram 
	for: 'SEIIRSInfluenzaRK';
	data: #(I1_species I2_species); "I2_Species"
	legendTitle: 'Total of Infectious';
	legends: #('birds strains 1' 'humans strains 1' 'birds strains 2' 'humans strains 2');
	xLabel: 'Time (days)'.! !

!TestsModel5 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/27/2016 17:36'!
before

		
	| model SEIRConcern SEIRSConcern twoStrains multiHostConcern spatialConcern simulator dB f|
	
	model := KEModel new.
	model population: (KEPopulation size: 27500).
	SEIRConcern := KEConcern new.
	SEIRConcern attributes: {#status->#(#S #E #I #R)}.
	SEIRConcern addParameters: { #beta. #gamma. #mu. #sigma}.
	SEIRConcern addParameter: #lambda value: 'beta*I/N'.
	SEIRConcern addEquations: { 
		'S:t=mu*N - lambda*S - mu*S'.
		'E:t=lambda*S - sigma*E - mu*E'.
		'I:t=sigma*E - gamma*I - mu*I'.
		'R:t=gamma*I - mu*R'
	 }.		
	
	SEIRSConcern := KEConcern dependOn: SEIRConcern.
	SEIRSConcern addTransitionFrom: { #status->#R } to: { #status->#S } probability: 'nu'.
	SEIRSConcern addParameter: #nu.
	
	twoStrains := KEConcern dependOn: SEIRConcern.
	twoStrains transitions: (SEIRConcern transitions 
		select: [:tr| ((tr from at: #status) = #I) | ((tr to at: #status) = #I) ]).
	twoStrains splitStatus: {  #(I). #strains->#(1 2) }.
	twoStrains addParameters: { #beta1. #beta2 }.
	
	multiHostConcern := KEConcern new.
	multiHostConcern addAttribute: #species value: #(#human #bird).
	
	spatialConcern := KEConcern new.
	spatialConcern addAttribute: #patch value: (1 to: 5) asArray.
	spatialConcern addParameter: #rho.
   (1 to: 5) do: [ :i| 
        (i < 5) 
        ifTrue: [  
            spatialConcern 
                addTransitionFrom: { #patch->i } 
                to: { #patch->(i+1) } 
                probability: [ :aModel| aModel atParameter: #rho ].
        ]
        ifFalse: [  
            spatialConcern 
                addTransitionFrom: { #patch->i } 
                to: { #patch->1 } 
                probability: [ :aModel| aModel atParameter: #rho ].
        ]
    ]. 
	
	model integrate: SEIRConcern.
	model integrate: SEIRSConcern.
	model integrate: twoStrains.
	model integrate: spatialConcern.
	model integrate: multiHostConcern.
	
	
	
	model addParameter: #beta1 value: [ :aModel| |c val| 
		c := aModel currentCompartment at: #species.
		c = #human ifTrue: [ val := #(0 0.21) ].
		c = #bird ifTrue: [ val := #(0 0.42) ].
		val
	].
	model addParameter: #beta2 value: [ :aModel| |c val| 
		c := aModel currentCompartment at: #species.
		c = #human ifTrue: [ val := #(0 0.021) ].
		c = #bird ifTrue: [ val := #(0 0.042) ].
		val
	].
	model atParameter: #lambda assignValue: [ :aModel| |c|
		c := aModel currentCompartment at: #patch.
		(((aModel atParameter: #beta1) *
		(aModel atCompartment: {#status->#I1. #patch->c})/(aModel atParameter: #N))
		+ ((aModel atParameter: #beta2) *
		(aModel atCompartment: {#status->#I2. #patch->c})/(aModel atParameter: #N))) sum.
	].
	model atParameter: #gamma assignValue: [ :aModel| |c1 c2 val| 
		c1 := aModel currentCompartment at: #species.
		c2 := aModel currentCompartment at: #status.
		(c1 = #human & c2 = #I1)  ifTrue: [ val := 0.25 ].
		(c1 = #human & c2 = #I2)  ifTrue: [ val := 0.0025 ].
		(c1 = #bird & c2 = #I1)  ifTrue: [ val := 0.233 ].
		(c1 = #bird & c2 = #I2)  ifTrue: [ val := 0.0023 ].
		val ].
	model atParameter: #sigma assignValue: [ :aModel| |c1 c2 val| 
		c1 := aModel currentCompartment at: #species.
		c2 := aModel nextCompartment at: #status.
		(c1 = #human & c2 = #I1)  ifTrue: [ val := 0.5 ].
		(c1 = #human & c2 = #I2)  ifTrue: [ val := 0.005 ].
		(c1 = #bird & c2 = #I1)  ifTrue: [ val := 0.67 ].
		(c1 = #bird & c2 = #I2)  ifTrue: [ val := 0.0067 ].
		val ].
	model atParameter: #mu assignValue: [ :aModel| |c val| 
		c := aModel currentCompartment at: #species.
		c = #human ifTrue: [ val := 0.000365 ].
		c = #bird ifTrue: [ val := 0.00137 ].
		val ].
	model atParameter: #N assignValue: [ :aModel| |c| 
		c := OrderedCollection new.
		c add: (aModel currentCompartment at: #patch).
		c add: (aModel currentCompartment at: #species).
		aModel sizeOfPopulation: (c asArray)
	].
	model atParameter: #nu assignValue: 0.00274.
	model atParameter: #rho assignValue: [ :aModel| |c val|
		c := aModel currentCompartment at: #species.
		c = #bird ifTrue: [ val := 0.1 ].
		c = #human ifTrue: [ val := 0.03 ].
		val
	].
	
	model 
		atCompartment: { #status->#S. #species->#bird. #patch->1 } 
		put: 4990
		atOthersPut: 0.
	model 
		atCompartment: { #status->#I1. #species->#bird. #patch->1 } 
		put: 9.
	model 
		atCompartment: { #status->#I2. #species->#bird. #patch->1 } 
		put: 1.
	2 to: 5 do: [ :i| 
		model 
			atCompartment: { #status->#S. #species->#bird. #patch->i } 
			put: 5000.
	].
	1 to: 5 do: [ :i| 
		model 
			atCompartment: { #status->#S. #species->#human. #patch->i } 
			put: 500 
	].
	
	simulator := KESimulator new: #RungeKutta from: 0.0 to: 500 step: 1.
	simulator executeOn: model.
	dB := KEDiagramBuilder new.
	f := [:name| |d tmp|
		tmp := (simulator timeSeriesAt: name) collect: [ :e| e value ].
		d := OrderedCollection new.
		1 to: tmp first data size do: [ :k| d add: (tmp collect: [:e| e data at: k]) sum ].
		(KETimeSeries from: d withIndex: tmp first index) compartment: (STON fromString: name)
		].
	dB data: { 
		(f value: '{#status:#I1,#species:#bird}'). 
		(f value: '{#status:#I1,#species: #human}').
		(f value: '{#status:#I2,#species:#bird}'). 
		(f value: '{#status:#I2,#species: #human}') }.

	dB xLabel: 'Time (days)'.
	dB legendTitle: 'Total of infectious'.
	dB legends: { 'birds strains 1'. 'humans strains 1'. 'birds strains 2'. 'humans strains 2' }.
	dB open.
	
	^ {simulator . model . dB}! !

!TestsModel5 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/11/2016 18:12'!
createModels

	| beforeList |
	beforeList := self before.
	beforeSimulator := beforeList first.
	beforeModel := beforeList second.
	beforeDiagram := beforeList third.
	self after.
	afterSimulator := Simulation withId: 'SEIIRSInfluenzaRK'.
	afterModel := KendrickModel withId: 'Influenza'.
	afterDiagram := (Visualization withId: 'SEIIRSInfluenzaViz') open.	! !

!TestsModel5 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/12/2016 16:54'!
script

KendrickModel Influenza.

Concern SIR
	attribute: #(status -> S I R);
	parameters: #(beta lambda gamma mu);
	lambda: #(beta*I/N);
	transitions: #(
		S -- lambda --> I.
		I -- gamma --> R.
		status -- mu --> Empty.
		Empty -- mu --> S.
	).	
		
Concern SEIRS
	extends: 'SIR';
	parameters: #(sigma nu);
	delay: #(sigma , S -- lambda --> I  , E);
	addTransition:	#(R -- nu --> S.);
	addTransition: #(E -- mu --> Empty.).

Concern SEIIRS
	extends: 'SEIRS';
	parameters: #(beta1 beta2);
	splitStatus: #(I strains #(1 2)).

Concern Biological
	attribute: #(species -> human bird).
	
Concern Demographical
	attribute: #(patch -> 1  2  3  4  5);
	parameters: #(rho);
	rho: 0.03;
	transitions: #(
		1 -- rho --> 2.
		2 -- rho --> 3.
		3 -- rho --> 4.
		4 -- rho --> 5.
		5 -- rho --> 1.).
	
Composition SEIIRSCombo
	model: 'Influenza';
	populationSize: 27500;
	concern: 'SEIIRS';
	concern: 'Demographical';
	concern: 'Biological';
	beta1_species: #(#(0 0.21) #(0 0.42));
	beta2_species: #(#(0 0.021) #(0 0.042));
	lambda: #(beta1*(I1_patch/N)+beta2*(I2_patch/N) sum);
	gamma_species_status: #(
		#(0 0 0.25 0.0025 0 0);
		#(0 0 0.233 0.0023 0 0);
	);
	sigma_species_status: #(
		#(0 0 0.5 0.005 0 0);
		#(0 0 0.67 0.0067 0);
	);
	mu_species: #(0.0000365 0.00137);
	N: #(species_patch);
	nu: 0.00274;		
	"delta_species: #(0.068 0.055);
	epsilon_species: #(0.096 0.082);"
	rho_species: #(0.03 0.1 0);
	S_species_patch: #(#(500 500 500 500 500) #(4990 5000 5000 5000 5000));
	I1_species_patch: #(#(0 0 0 0 0) #(9 0 0 0 0));
	I2_species_patch: #(#(0 0 0 0 0) #(1 0 0 0 0)).

Simulation SEIIRSInfluenzaRK rungeKutta
	for: 'SEIIRSCombo';
	from: 0; 
	to: 500; 
	step: 1.
	
Visualization SEIIRSInfluenzaViz diagram 
	for: 'SEIIRSInfluenzaRK';
	data: #(I2_species); "I2_Species"
	legendTitle: 'Total of Infectious';
	legends: #('birds strains 1' 'humans strains 1' 'birds strains 2' 'humans strains 2');
	xLabel: 'Time (days)'.! !

!TestsModel5 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/11/2016 18:11'!
testEquivalence


	| resultsBefore resultsAfter |
	self createModels.

	beforeModel inspect.
	afterModel keModel inspect.

	"correct (check inspector) -- but timeseries are presented in different order fix"

	resultsBefore := (beforeDiagram data collect: [ :ts | ts peakOfEpidemic floor ]) sorted asOrderedCollection.
	resultsAfter := (afterDiagram data collect: [ :ts | ts peakOfEpidemic floor ]) sorted asOrderedCollection.
	Transcript show: resultsBefore; cr; show: resultsAfter; cr; cr.
	self assert: resultsBefore = resultsAfter.! !

!TestsModel5 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/11/2016 18:11'!
testScript! !

!TestsModel5 methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/11/2016 16:05'!
vizId

	^ 'SEIIRSInfluenzaViz'! !

!KEModel methodsFor: '*ExtentionsForKendrick' stamp: 'NickPapoulias 4/13/2016 17:21'!
resetTime

	t := 0! !

Error subclass: #KendrickDSLSyntaxError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

KEGillespieSimulator subclass: #KEGillespieSimulatorForTesting
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

KEGillespieSimulatorForTesting class
	instanceVariableNames: 'seed'!

KEGillespieSimulatorForTesting class
	instanceVariableNames: 'seed'!

!KEGillespieSimulatorForTesting class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/13/2016 16:31'!
newSeed

	seed :=  (Random new next * 1000000000) asInteger -> (Random new next * 1000000000) asInteger! !

!KEGillespieSimulatorForTesting class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/12/2016 16:52'!
randPair

	^ (Random new seed: seed key) -> (Random new seed: seed value)! !

!KEGillespieSimulatorForTesting class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/12/2016 16:13'!
seed

	^ seed! !

!KEGillespieSimulatorForTesting class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/12/2016 17:46'!
seed: aPair

	seed := aPair! !

!KEGillespieSimulatorForTesting methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/12/2016 17:37'!
executeOn: aModel
	| rand1 rand2 t previousT randPair |
	super executeOn: aModel.
	t := tMin.
	previousT := t.
	randPair := self class randPair. "new generators with same seed (re-new controlled from class side)"
	rand1 := randPair key.
	rand2 := randPair value.
	self saveCompartmentsValueAtTime: tMin.

	[ t < tMax ]
		whileTrue: [ 
			| deltaT r1 r2 |
			r1 := rand1 next.
			r2 := rand2 next.
			deltaT := self doGillespieIteration: t randoms: (Array with: r1 with: r2).
			(deltaT = 0.0)
				ifTrue: [ ^ self ].
			t := t + deltaT.
			"Only store in the result from step to step"
			t > (previousT + step)
				ifTrue: [ 
					self saveCompartmentsValueAtTime: t.
					previousT := t ] ]! !

Object subclass: #BenchExamples
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!BenchExamples class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/19/2016 21:58'!
run

	Transcript show: 'Measles'; cr.
	TestMeaslesScripts new bench.
	Transcript show: 'Infuenza 1SP'; cr.
	TestDependantConcern new bench.
	Transcript show: 'Infuenza 2SP'; cr.
	TestMultiSpeciesSpatialDependant new bench.! !

Object subclass: #KendrickEquations
	instanceVariableNames: 'eqAsSymExpr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!KendrickEquations methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 13:05'!
convertToCompatibleBackendRepresentation

	^ ((eqAsSymExpr asString
			copyReplaceAll: '#' with: '' asTokens: false)
			trimBoth: [ :c | (c = $() | (c = $)) ]) 
			findTokens: '''.'''! !

!KendrickEquations methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/4/2016 17:57'!
with: symExpression

	eqAsSymExpr := symExpression ! !

Object subclass: #KendrickInternalDSLEntities
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

KendrickInternalDSLEntities class
	instanceVariableNames: 'storedInstances'!

KendrickInternalDSLEntities class
	instanceVariableNames: 'storedInstances'!

!KendrickInternalDSLEntities class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:23'!
clearAll

	self allSubclassesDo: [ :subclass | subclass clearAllInstances ]! !

!KendrickInternalDSLEntities class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:20'!
clearAllInstances

	storedInstances := nil! !

!KendrickInternalDSLEntities class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/18/2016 17:59'!
doesNotUnderstand: aMessage

	^ self new id: aMessage selector! !

!KendrickInternalDSLEntities class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/27/2016 14:00'!
entityFilesDir

	^ Smalltalk vmDirectory asFileReference ! !

!KendrickInternalDSLEntities class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:22'!
new

	^ self storedInstances addFirst: self basicNew initialize! !

!KendrickInternalDSLEntities class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 16:20'!
storedInstances

	^ storedInstances ifNil: [ storedInstances := OrderedCollection new ]! !

!KendrickInternalDSLEntities class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/31/2016 20:38'!
"protocol: as yet unclassified"
withId: anId
	
	|inImage inDisk|

	inImage := (self storedInstances select: [ :instance | instance id = anId ]).
	inImage isEmpty 	
		ifFalse: [ ^ inImage first ]
		ifTrue: [  
			inDisk := self entityFilesDir / self asString / (anId asString , '.kendrick').
			inDisk exists 	
				ifTrue: [ 
					|rs entity|
					rs :=  inDisk readStream. 
					entity := Compiler evaluate: rs upToEnd.
					rs close.  
					^ entity
				]
				ifFalse: [ Error signal: 'NO such entity: ' , anId asString ]	
		]! !

!KendrickInternalDSLEntities methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/11/2016 16:31'!
doesNotUnderstand: aMessage

	KendrickDSLSyntaxError signal: self class name , ' has no keyword ' , aMessage selector.! !

!KendrickInternalDSLEntities methodsFor: 'public-api' stamp: 'NickPapoulias 4/26/2016 17:20'!
extends: anotherEntityName
	"copy down semantics -- check how shallow is the default copy on this one"
	|previousId|
	previousId := id.
	self become: (self class withId: anotherEntityName) copy.
	self id: previousId.! !

!KendrickInternalDSLEntities methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/4/2016 16:51'!
id

	^ id! !

!KendrickInternalDSLEntities methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/4/2016 16:52'!
id: aString

	id := aString! !

KendrickInternalDSLEntities subclass: #KendrickModelingComponent
	instanceVariableNames: 'attributes parameters equations transitions explicitelySetCompartments otherCompartmentValue emptyRate backEndEntity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

KendrickModelingComponent subclass: #KendrickComposite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

KendrickComposite subclass: #Composition
	instanceVariableNames: 'model concerns compartments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/11/2016 18:08'!
addCompositeAtrributeOrParameter: variable value: value

	|variablePlusAttributes|
	
	variablePlusAttributes := variable asString findTokens: '_'.
	"assume single attribute for the time being"
	
	(model attributeValuesIncludes: variablePlusAttributes first asSymbol)
		ifTrue: [ 
			self addCompositeAttribute: variablePlusAttributes value: value. 
		] ifFalse: [  
		(model parameters keys includes: variablePlusAttributes first asSymbol)
			ifTrue: [  
				self addCompositeParameter: variablePlusAttributes value: value.
			] ifFalse: [ 
					KendrickDSLSyntaxError 
								signal: 'Undefined Variable Being Set: ' , variable asString ]. 
		].
	! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/17/2016 16:23'!
addCompositeAttribute: variablePlusAttributes value: value

	compartments at: variablePlusAttributes put: value.
	(self allCompositeReferencesFor: variablePlusAttributes)
		doWithIndex: [:ref :index|  
			(value anySatisfy: [ :item | item isCollection ])
				ifTrue: [ backEndEntity atCompartment: ref put: (value flatten at: index) ]
				ifFalse: [ backEndEntity atCompartment: ref put: (value at: index) ]		
		]! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/27/2016 20:10'!
addCompositeParameter: variablePlusAttributes value: value

	|entities entity |

	"should be generilized"
	
	entities := self entitiesFor: variablePlusAttributes allButFirst.
	
	"Refactor this one ..."
	
	parameters at: variablePlusAttributes put: value.
	
	((variablePlusAttributes size >= 3) and: [ entities second isEmpty ])
		ifTrue: [ ^ self addCompositeParameterWithTwoAttributes: variablePlusAttributes value: value ].

	
	backEndEntity 
        addParameter: variablePlusAttributes first asSymbol
        value: 
        [ :aModel| |c index attrEnt compEnt|
				attrEnt := entities first first. "for now suppose you have one of each"
				compEnt := entities second. "or only attributes"
            c := aModel currentCompartment at: variablePlusAttributes second.
				index := (attrEnt attributes at: variablePlusAttributes second) indexOf: c.            
				compEnt isEmpty
					ifTrue: [value at: index]
					ifFalse: [  
						c := aModel currentCompartment at: compEnt first attributes keys first. "should pick the right attr here"
						c = variablePlusAttributes last 
							ifTrue: [ value last ]
							ifFalse: [ 
								value at: index.
							]
					]  
        ].

! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/27/2016 20:16'!
addCompositeParameterWithTwoAttributes: variablePlusAttributes value: value

	|entities entityForSecond entityForThird |
	
	entityForSecond := (self entitiesFor: {variablePlusAttributes second}) first first.
	entityForThird := (self entitiesFor: {variablePlusAttributes third}) first first. 

	backEndEntity 
        addParameter: variablePlusAttributes first asSymbol
        value: [ :aModel| |c1 c2 index1 index2| 
			c1 := aModel currentCompartment at: variablePlusAttributes second.
			variablePlusAttributes last asSymbol = 'next' 
				ifFalse: [c2 := aModel currentCompartment at: variablePlusAttributes third]
				ifTrue: [c2 := aModel nextCompartment at: variablePlusAttributes third].
			index1 := (entityForSecond attributes at: variablePlusAttributes second) indexOf: c1.
			index2 := (entityForThird attributes at: variablePlusAttributes third) indexOf: c2.
			(value at: index1) at: index2.
	 ].

! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/24/2016 17:19'!
allCompositeReferencesFor: variablePlusAttributes

	|attributes compartment entities compResults|
	compartment := variablePlusAttributes first.	
	attributes := variablePlusAttributes allButFirst 
						collect: [ :attribute |  
							((concerns copy add: model; yourself) 
								select: [ :e | e attributes keys includes: (attribute)]) first attributes associationAt: attribute ].
	
	compResults := {{ self referencesFor: compartment } asOrderedCollection } asOrderedCollection.
	
	attributes do: [ :attribute | 
		|newCompResults|
		newCompResults := OrderedCollection new.
		compResults do: [ :resultList | 
			attribute value do: [ :value |
					newCompResults add: (resultList copy add: attribute key -> value; yourself).
			]
		].
		compResults := newCompResults.
	].
	
	
	^ compResults collect: [ :res | res asArray ]
	! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/27/2016 11:44'!
concern: aString
	|concern|

	concern := Concern withId: aString.
	(concerns includes: concern) ifFalse: [  "no duplicates"
		concern extends do: [ :ext | 
			self concern: ext 
		]. "-- is this integrated in the backend ?" 
		concerns add: concern.
		model integrate: concern.
	]! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/27/2016 12:50'!
doesNotUnderstand: aMessage

	|variable value|

	"TODO: should also search in the concerns for setting comp/attri"
	
	variable := aMessage selector.
	variable := (variable asString trimRight: [: c | c =  $:]) asSymbol.
	value := aMessage arguments first.
 
	(model attributeValuesIncludes: variable)
		ifTrue: [ 
			explicitelySetCompartments add: variable.
			compartments at: variable put: value.
			model atCompartment: variable value: value. 
		] ifFalse: [  
		(model parameters keys includes: variable)
			ifTrue: [  
				parameters at: variable put: value.
				model atParameter: variable value: value. "can be array here treat differently"
			] ifFalse: [ 
					(aMessage selector asString includes: $_)
						ifTrue: [ self addCompositeAtrributeOrParameter: variable value: value ]
						ifFalse: [  
							KendrickDSLSyntaxError 
								signal: 'Undefined Variable Being Set: ' , variable asString
						] 
					]. 
		].! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/29/2016 15:54'!
entitiesFor: attr 

	"first list entities with attributes, second list entities with values"

	^ { 
	
		(attr collect: [:attribute | 
			((concerns copy add: model; yourself) 
				select: [ :e | 
					e attributes keys includes: attribute
	 			]
			)  ]) flatten.
			
		(attr collect: [:attribute | 
			((concerns copy add: model; yourself) 
				select: [ :e | 
					e attributes values first includes: attribute
	 			]
			) ]) flatten.	
			
	}	
! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/27/2016 12:49'!
initialize

	super initialize.
	concerns := OrderedCollection new. 
	compartments := Dictionary new. ! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/17/2016 18:41'!
model

	^ model! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/17/2016 16:23'!
model: aString

	model := KendrickModel withId: aString.
	backEndEntity := model keModel ! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/27/2016 12:40'!
others: anInteger

	^ model others: anInteger given: explicitelySetCompartments. "TODO: propagate to concerns as well"

	! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/18/2016 12:32'!
populationSize: aNumber

	model population: aNumber
	! !

!Composition methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/11/2016 17:55'!
referencesFor: anAttribute

	|assoc|

	assoc := (model attributes 
		associations select: [ :as | as value includes: anAttribute ]) first.
	^ assoc key asSymbol -> anAttribute asSymbol 

	! !

KendrickComposite subclass: #Concern
	instanceVariableNames: 'delays structuralDependencies extending'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 5/29/2016 15:16'!
addStatus: symExpression

	(attributes at: #status) add: symExpression first.
	backEndEntity addStatus: symExpression.! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 18:22'!
addTransition: symExpression

	|newTransition| 

	newTransition :=
		KendrickTransitions new with: symExpression.
	newTransition convertToCompatibleBackendRepresentation do: [:assoc |
		|from to propability|
		from := assoc key key asString asSymbol.
		propability := assoc key value.
		to := assoc value asString asSymbol.
		self allTransitionsFor: { from . propability . to } ].! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 6/27/2016 13:32'!
allHierarchyTransitions

	^ backEndEntity transitions copy 
		addAll: (structuralDependencies 
			flatCollect: [ :name | (Concern withId: name) allHierarchyTransitions ]);
		yourself! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 17:52'!
delay: symExpression
 
	"#(sigma , S -- lambda --> I  , E)"
	| varList delay targetLeftNode initialRate targetRightNode newNode |
	varList := symExpression 
		select: [ :token | (#(-- --> ,) includes: token) not ].
	delays add: varList.
	delay := varList first.
	targetLeftNode := self literalTransitionRefFor: varList second.
	initialRate := varList third.
	targetRightNode :=  self literalTransitionRefFor: varList fourth.
	newNode := { targetRightNode first key -> varList fifth }.
	
	backEndEntity atAttribute: targetRightNode first key add: varList fifth.
	(attributes at: targetRightNode first key) add: varList fifth.
	
	backEndEntity 
		removeTransitionFrom: targetLeftNode  
		to: targetRightNode.
	backEndEntity 
		addTransitionFrom: targetLeftNode  
		to: newNode  
    	probability: [ :aModel| aModel atParameter: initialRate ].
	backEndEntity 
    addTransitionFrom: newNode  
    to: targetRightNode  
    probability: [ :aModel| aModel atParameter: delay ].
	backEndEntity 
    addTransitionFrom: newNode 
    to: #empty 
    probability: [ :aModel| aModel atParameter: (emptyRate ifNil: [#mu]) "default value" ].! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 18:29'!
doesNotUnderstand: aMessage

	|variable value|
	
	variable := aMessage selector.
	variable := (variable asString trimRight: [: c | c =  $:]) asSymbol.
	value := aMessage arguments first.
 
	(self attributeValuesIncludes: variable)
		ifTrue: [ 
			explicitelySetCompartments add: {variable->value}.
		] ifFalse: [  
		(parameters keys includes: variable)
			ifTrue: [  
				parameters at: variable put: value.
			] ifFalse: [ 
					KendrickDSLSyntaxError 
								signal: 'Undefined Variable Being Set: ' , variable asString ]. 
		].
	
	! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 15:49'!
extends

	^ structuralDependencies
! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 6/27/2016 13:00'!
extends: anotherConcernName

	super extends: anotherConcernName.
	structuralDependencies := {  anotherConcernName }.
	transitions := OrderedCollection new.
	extending := backEndEntity.
	backEndEntity := KEConcern dependOn: backEndEntity.! !

!Concern methodsFor: 'protected' stamp: 'NickPapoulias 5/27/2016 12:47'!
initialize

	super initialize.

	backEndEntity := KEConcern new.
	structuralDependencies := OrderedCollection new.
	delays := OrderedCollection new.
! !

!Concern methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 16:22'!
keConcern

	^ backEndEntity.
! !

!Concern methodsFor: 'public-api' stamp: 'NickPapoulias 6/27/2016 16:56'!
splitStatus: symExpression

	"splitStatus: {  #(I). #strains->#(1 2) }.
	splitStatus: #(I strains #(1 2))"

	|statusCompartment indexes|
	
	statusCompartment := symExpression first.
	indexes := symExpression third.

	indexes do: [:index | 
		(attributes at: #status) 
			add: (statusCompartment asString , index asString) asSymbol.
	].

	backEndEntity transitions: (self allHierarchyTransitions 
		select: [:tr| ((tr from at: #status) = statusCompartment) | ((tr to at: #status) = statusCompartment) ]).
	backEndEntity splitStatus: { { statusCompartment } asArray . symExpression second -> indexes }.! !

KendrickModelingComponent subclass: #KendrickModel
	instanceVariableNames: 'population'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!KendrickModel methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 16:20'!
addParameter: variable value: value

	value isNumber 
		ifTrue: [backEndEntity addParameter: variable value: value]
		ifFalse: [ "array" 
			backEndEntity 
				addParameter: variable 
				value: (self covertSymbolicExpressionToBlock: value).
		]! !

!KendrickModel methodsFor: 'protected' stamp: 'NickPapoulias 4/12/2016 14:35'!
allTransitionRefsForAttributeWith: aCompartment
	
	|assoc|

	aCompartment asString translateToLowercase = 'empty'
		ifTrue: [ ^ { #empty } ].
	
	assoc := (attributes associations select: [ :pair | pair value includes: aCompartment ]).
	^ assoc value collect: [:comp | '{#' , assoc key asString , ': #' , comp , '}']! !

!KendrickModel methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 16:20'!
atCompartment: variable value: value

	|assoc|

	assoc := (attributes associations select: [ :as | as value includes: variable ]) first.
	backEndEntity atCompartment: {assoc key -> variable} put: value
	! !

!KendrickModel methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 16:20'!
atParameter: variable value: value

	value isNumber 
		ifTrue: [backEndEntity atParameter: variable assignValue: value]
		ifFalse: [ "array" 
			backEndEntity 
				atParameter: variable 
				assignValue: (self covertSymbolicExpressionToBlock: value).
		]! !

!KendrickModel methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 16:20'!
attributeOrParameter: aName

	^ (parameters keys includes: aName)
		ifTrue: [ backEndEntity atParameter: aName ]
		ifFalse: [  
			|assoc|
			assoc := (attributes associations select: [ :as | as value includes: aName ]).
			assoc isEmpty 
				ifTrue: [ 
					(aName asString includes: $_)
							ifTrue: [ |tokens c|
								tokens := aName findTokens: '_'.
								(attributes keys includes: tokens first)
									ifFalse: [  
										c := backEndEntity currentCompartment at: tokens second.
										assoc := (attributes associations select: [ :as | as value includes: tokens first ]).
										backEndEntity atCompartment: { assoc first key -> tokens first asSymbol . tokens second -> c } 
									] ifTrue: [  
										backEndEntity sizeOfPopulation: (tokens collect: [:t | (backEndEntity currentCompartment at: t)]) asArray.
									]
							]
							ifFalse: [backEndEntity sizeOfPopulation: (backEndEntity currentCompartment at: aName asSymbol)] 
				] "assume attribute name"
				ifFalse: [
								backEndEntity atCompartment: { assoc first key -> aName }
				]
		]
	! !

!KendrickModel methodsFor: 'protected' stamp: 'NickPapoulias 5/29/2016 20:04'!
covertSymbolicExpressionToBlock: anArray

			|body|

			body := (anArray collect: [ :token | 
				self tokenAsString: token
			]) inject: '' into: [ :expr :token | 
				(#(sum sqrt size) includes: token)
					ifFalse: [expr , ' ' , token]
					ifTrue: [ '(' , expr , ')' , token  ]
			].
			"the argument is superficial here since it is captured (backward compatibility)"
	
			^ Compiler new		evaluate: '[:aModel |' , body , ']'
					in: thisContext sender
					to: thisContext sender receiver
					notifying: nil
					ifFail: [^nil]
					logged: false.! !

!KendrickModel methodsFor: 'public-api' stamp: 'NickPapoulias 4/11/2016 14:32'!
doesNotUnderstand: aMessage

	|variable value|
	
	variable := aMessage selector.
	variable := (variable asString trimRight: [: c | c =  $:]) asSymbol.
	value := aMessage arguments first.
 
	(self attributeValuesIncludes: variable)
		ifTrue: [ 
			explicitelySetCompartments add: variable.
			self atCompartment: variable value: value. 
		] ifFalse: [  
		(parameters keys includes: variable)
			ifTrue: [  
				parameters at: variable put: value.
				self addParameter: variable value: value. "can be array here treat differently"
			] ifFalse: [ 
					KendrickDSLSyntaxError 
								signal: 'Undefined Variable Being Set: ' , variable asString ]. 
		].
	
	! !

!KendrickModel methodsFor: 'protected' stamp: 'NickPapoulias 5/27/2016 12:46'!
initialize

	super initialize.
	backEndEntity := KEModel new.
	parameters at: #N put: nil. "seems to be a default one"! !

!KendrickModel methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 16:20'!
integrate: concern.

	backEndEntity integrate: concern keConcern.
	parameters addAll: concern parameters.
	concern parameters keysAndValuesDo: [ :key :value | self addParameter: key value: value ].
	attributes addAll: concern attributes. "init compartments from concerns not possible just add"
	
! !

!KendrickModel methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 16:20'!
keModel

	^ backEndEntity! !

!KendrickModel methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 16:20'!
others: anInteger

	otherCompartmentValue := anInteger.

	backEndEntity attributes
		associationsDo: [ :assoc | 
			assoc value do: [:var |
				(explicitelySetCompartments includes: var)
					ifFalse: [ self atCompartment: var value: anInteger ]
			] 
		]! !

!KendrickModel methodsFor: 'public-api' stamp: 'NickPapoulias 5/27/2016 12:40'!
others: anInteger given: explicitelySetCompInOtherEntity

	explicitelySetCompartments addAll: explicitelySetCompInOtherEntity.
	self others: anInteger! !

!KendrickModel methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 16:20'!
population: anInteger

	population := anInteger.
	backEndEntity population: (KEPopulation size: anInteger).! !

!KendrickModel methodsFor: 'protected' stamp: 'NickPapoulias 5/29/2016 20:01'!
tokenAsString: token

 token isArray ifTrue: [ 
		^ (token 
			inject: '( ' 
			into: [:subTotal :t | subTotal , ' ' ,  (self tokenAsString: t) ]) , ')'
	].

 ^ (#(- + * / , sum sqrt) includes: token)
				ifTrue: [ token asString ]
				ifFalse: [ 
						 '(self attributeOrParameter: ' ,  '#' , token asString , ')' 
				]			! !

!KendrickModelingComponent methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 16:05'!
allTransitionRefsForAttribute: attribute
	
	^ (attributes at: attribute) collect: [:comp | '{#' , attribute asString , ': #' , comp , '}']! !

!KendrickModelingComponent methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 16:29'!
allTransitionsFor: aFromToList

		|from to propability|

		from := aFromToList first.
		propability := aFromToList second.
		to := aFromToList last.
		({ from . to } includes: #Empty) ifTrue: [ emptyRate := propability ].
		(self isAttribute: from)
			ifTrue: [ 
				(self allTransitionRefsForAttribute: from) do: [:fromRef |  
					backEndEntity
   						addTransitionFrom: fromRef
   						to: (self transitionRefFor: to)
   						probability: [ :m | m atParameter: propability ].			
				]
			]
			ifFalse: [ "to = all"
				(self isAttribute: to)
					ifTrue: [  
						(self allTransitionRefsForAttribute: from) do: [:toRef |  
							backEndEntity
   								addTransitionFrom: (self transitionRefFor: from)
   								to: toRef
   								probability: [ :m | m atParameter: propability ].			
						]
					] ifFalse: [ "no attributes"  
						backEndEntity
   								addTransitionFrom: (self transitionRefFor: from)
   								to: (self transitionRefFor: to)
   								probability: [ :m | m atParameter: propability ].
					]		
			 ] 
			
	! !

!KendrickModelingComponent methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 17:49'!
attribute: anArray

	attributes 
		at: anArray first 
		put: ((anArray allButFirst select: [:var | var ~= #->]) collect: [ :var | var asString asSymbol]) asOrderedCollection.
	backEndEntity 
		addAttribute: anArray first 
		value: ((anArray allButFirst select: [:var | var ~= #->]) collect: [ :var | var asString asSymbol]). 
	! !

!KendrickModelingComponent methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 16:32'!
attributeValuesIncludes: variable

	^ (attributes values select: [ :values | values includes: variable ]) isEmpty not! !

!KendrickModelingComponent methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 16:18'!
attributes

	^ attributes! !

!KendrickModelingComponent methodsFor: 'public-api' stamp: 'NickPapoulias 5/29/2016 19:32'!
equations: anArrayOfSymExpressions

	equations :=
		KendrickEquations new with: anArrayOfSymExpressions.
	backEndEntity addEquations: equations convertToCompatibleBackendRepresentation.
	! !

!KendrickModelingComponent methodsFor: 'public-api' stamp: 'NickPapoulias 5/27/2016 12:46'!
initialize
	super initialize.
	attributes := Dictionary new.
	parameters := Dictionary new.
	explicitelySetCompartments := OrderedCollection new.
		! !

!KendrickModelingComponent methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 16:18'!
isAttribute: attribute
	
	^ (attributes keys includes: attribute) ! !

!KendrickModelingComponent methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 16:19'!
literalTransitionRefFor: aCompartment
	
	|assoc|

	
	aCompartment = 'Empty'
		ifTrue: [ ^ #empty ].
	assoc := (attributes associations select: [ :pair | pair value includes: aCompartment ]) first.
	^ {   assoc key -> aCompartment }.
	! !

!KendrickModelingComponent methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 16:28'!
others: anInteger

	otherCompartmentValue := anInteger.
		! !

!KendrickModelingComponent methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 16:18'!
parameters

	^ parameters ! !

!KendrickModelingComponent methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 16:07'!
parameters: anArray

	anArray do: [ :var | parameters at: var put: 0 ]! !

!KendrickModelingComponent methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 16:31'!
transitionRefFor: aCompartment
	
	|assoc|

	
	aCompartment = 'Empty'
		ifTrue: [ ^ #empty ].
	assoc := (attributes associations select: [ :pair | pair value includes: aCompartment ]) first.
	^ '{#' , assoc key asString , ': #' , aCompartment , '}'.! !

!KendrickModelingComponent methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 16:08'!
transitions: aSymExpression

	transitions :=
		KendrickTransitions new with: aSymExpression.
	transitions convertToCompatibleBackendRepresentation do: [:assoc |
		|from to propability|
		from := assoc key key asString asSymbol.
		propability := assoc key value.
		to := assoc value asString asSymbol.
		self allTransitionsFor: { from . propability . to } ].
		! !

KendrickInternalDSLEntities subclass: #Simulation
	instanceVariableNames: 'algo from to step simulator combo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!Simulation methodsFor: 'protected' stamp: 'NickPapoulias 4/5/2016 15:38'!
allTimeSeries

	^ simulator allTimeSeries ! !

!Simulation methodsFor: 'protected' stamp: 'NickPapoulias 5/17/2016 18:35'!
execute

	simulator executeOn: combo model keModel.! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 18:34'!
for: aComboID

	combo := Composition withId: aComboID.! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 18:49'!
forModel: aModelID

	combo := Composition dummy model: aModelID.! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:27'!
from: aNumber

	from := aNumber! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/18/2016 17:51'!
gillespie

	^ self  
		with: #Gillespie! !

!Simulation methodsFor: 'protected' stamp: 'NickPapoulias 4/12/2016 16:33'!
init

	algo = #testGillespie
		ifTrue: [ simulator := (KEGillespieSimulatorForTesting new). 
							simulator tMin: from; tMax: to; step: step ]
		ifFalse: [  simulator := KESimulator new: algo from: from to: to step: step ]! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 5/17/2016 18:35'!
model

	^ combo model! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/18/2016 17:51'!
rungeKutta

	^ self  
		with: #RungeKutta! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:29'!
step: aNumber

	step := aNumber! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/18/2016 17:50'!
testGillespie

	^ self  
		with: #testGillespie! !

!Simulation methodsFor: 'protected' stamp: 'NickPapoulias 4/11/2016 18:13'!
timeSeriesAt: anIdentifier

	^ simulator timeSeriesAt: anIdentifier ! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:27'!
to: aNumber

	to := aNumber! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:24'!
with: anAlgorithm

	algo := anAlgorithm ! !

KendrickInternalDSLEntities subclass: #Visualization
	instanceVariableNames: 'simulation xLabel diag builder explicitDataSet lazyDataBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 6/28/2016 13:06'!
data: anArray "should be full math-expression"

	"refactor !!!!"

	explicitDataSet := true.

	(#(sum sqrt size) includes: anArray last) ifTrue: [  
			self dataWithOp: anArray 
	] ifFalse: [  
			self dataWithCompartments: anArray		
	]! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 6/28/2016 13:07'!
dataWithCompartments: anArray "should be full math-expression"

	|tSeries|
		
		^ lazyDataBlock := [		
				(anArray first asString includes: $_)
						ifTrue: [self dataWithCompositeCompartments: anArray]
						ifFalse: [self dataWithSimpleCompartments: anArray]
			]
					
			! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 6/28/2016 13:10'!
dataWithCompositeCompartments: anArray "should be full math-expression"
		
		| tokens tSeries |
		
		tSeries := OrderedCollection new.
		
		anArray do: [:name | 
				tokens := name findTokens: '_'.
					(simulation model allTransitionRefsForAttribute: tokens second) do: [:comp |  
						|compositeCompartment|
						compositeCompartment 
							:= (simulation model transitionRefFor: tokens first) allButLast.
						compositeCompartment := compositeCompartment , ' , ' , comp allButFirst.
						tSeries add: (self sumOverTimeSeries: compositeCompartment).			
					]	
				].

	^ diag data: tSeries.
					
			! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 6/28/2016 12:41'!
dataWithOp: anArray "should be full math-expression"

	^ lazyDataBlock := [
			diag data: 
				((simulation 
					timeSeriesAt: 
						(simulation model transitionRefFor: anArray first))
					perform: anArray last)
			]! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 6/28/2016 13:01'!
dataWithSimpleCompartments: anArray "should be full math-expression"

		^ diag data: 
			(simulation 
					timeSeriesAt: 
						(simulation model transitionRefFor: anArray first))

					
			! !

!Visualization methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/18/2016 17:58'!
diagram

	^ self 
		with: KEDiagramBuilder! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 4/11/2016 18:15'!
for: simulationID

	simulation := (Simulation withId: simulationID) init.! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 4/11/2016 18:16'!
initialize

	explicitDataSet := false! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 4/19/2016 18:19'!
legendTitle: aString

	diag legendTitle: aString! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 5/7/2016 14:29'!
legends: anArray

	diag legends: anArray asOrderedCollection ! !

!Visualization methodsFor: 'protected' stamp: 'NickPapoulias 5/19/2016 18:51'!
open

	simulation execute.
	explicitDataSet 
		ifFalse: [ diag data: simulation allTimeSeries ]
		ifTrue: [ lazyDataBlock value ].
	diag xLabel: xLabel.
	diag open.
	^ diag! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 4/24/2016 18:32'!
sumOverTimeSeries: name

	"refactor !!!!"
	|d tmp|
    tmp := (simulation timeSeriesAt: name) collect: [ :e| e value ].
    d := OrderedCollection new.
    1 to: tmp first data size do: [ :k| d add: (tmp collect: [:e| e data at: k]) sum ].
    ^ (KETimeSeries from: d withIndex: tmp first index) compartment: (STON fromString: name)
   ! !

!Visualization methodsFor: 'protected' stamp: 'NickPapoulias 4/12/2016 09:35'!
with: aBuilder

	builder := aBuilder.
	diag := builder new.! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:36'!
xLabel: aString

	xLabel := aString! !

Object subclass: #KendrickTransitions
	instanceVariableNames: 'eqAsSymExpr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!KendrickTransitions methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/18/2016 19:19'!
convertToCompatibleBackendRepresentation

	|transitions currentTransition|

	transitions := OrderedCollection new.
	currentTransition := nil.

	eqAsSymExpr  do: [ :token | 
		(token = #--> or: [ token = #-- ])
			ifTrue: [  
				"do nothing"
			] ifFalse: [ 
				token = #'.'
					ifTrue: [ transitions add: currentTransition. currentTransition := nil ]
					ifFalse: [ 
						currentTransition = nil 
							ifTrue: [ currentTransition := token ]
							ifFalse: [ currentTransition := currentTransition -> token ]  
					]
			]
	].

	^ transitions 

	! !

!KendrickTransitions methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/11/2016 11:30'!
with: symExpression

	eqAsSymExpr := symExpression ! !

!Object methodsFor: '*ExtentionsForKendrick' stamp: 'NickPapoulias 4/24/2016 16:11'!
, anotherObject

	^ { self . anotherObject } asArray! !

!BlockClosure methodsFor: '*ExtentionsForKendrick' stamp: 'NickPapoulias 5/29/2016 20:09'!
expression

	^ self! !

!BlockClosure methodsFor: '*ExtentionsForKendrick' stamp: 'NickPapoulias 5/29/2016 20:06'!
isAVariable

	^ true! !

!BlockClosure methodsFor: '*ExtentionsForKendrick' stamp: 'NickPapoulias 5/29/2016 20:07'!
isUnaryExpression

	^ true! !

!BlockClosure methodsFor: '*ExtentionsForKendrick' stamp: 'NickPapoulias 5/29/2016 20:06'!
symbol

	^ self at: 1! !
